from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Alias, Bool, Integer, MinMax, NoneSet, Set, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedNoneSet, NestedValue, _NestedNoneSetParam
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.colors import ColorChoice, HSLColor, RGBPercent as _RGBPercent, SchemeColor, SystemColor, _PresetColors
from openpyxl.drawing.effect import (
    AlphaBiLevelEffect,
    AlphaCeilingEffect,
    AlphaFloorEffect,
    AlphaInverseEffect,
    AlphaModulateEffect,
    AlphaModulateFixedEffect,
    AlphaReplaceEffect,
    BiLevelEffect,
    BlurEffect,
    ColorChangeEffect,
    ColorReplaceEffect,
    DuotoneEffect,
    FillOverlayEffect,
    GrayscaleEffect,
    HSLEffect,
    LuminanceEffect,
    TintEffect,
)

from ..xml._functions_overloads import _HasTagAndGet

_PatternFillPropertiesPrst: TypeAlias = Literal[
    "pct5",
    "pct10",
    "pct20",
    "pct25",
    "pct30",
    "pct40",
    "pct50",
    "pct60",
    "pct70",
    "pct75",
    "pct80",
    "pct90",
    "horz",
    "vert",
    "ltHorz",
    "ltVert",
    "dkHorz",
    "dkVert",
    "narHorz",
    "narVert",
    "dashHorz",
    "dashVert",
    "cross",
    "dnDiag",
    "upDiag",
    "ltDnDiag",
    "ltUpDiag",
    "dkDnDiag",
    "dkUpDiag",
    "wdDnDiag",
    "wdUpDiag",
    "dashDnDiag",
    "dashUpDiag",
    "diagCross",
    "smCheck",
    "lgCheck",
    "smGrid",
    "lgGrid",
    "dotGrid",
    "smConfetti",
    "lgConfetti",
    "horzBrick",
    "diagBrick",
    "solidDmnd",
    "openDmnd",
    "dotDmnd",
    "plaid",
    "sphere",
    "weave",
    "divot",
    "shingle",
    "wave",
    "trellis",
    "zigZag",
]
_PropertiesFlip: TypeAlias = Literal["x", "y", "xy"]
_TileInfoPropertiesAlgn: TypeAlias = Literal["tl", "t", "tr", "l", "ctr", "r", "bl", "b", "br"]
_BlipCstate: TypeAlias = Literal["email", "screen", "print", "hqprint"]
_PathShadePropertiesPath: TypeAlias = Literal["shape", "circle", "rect"]

class PatternFillProperties(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    prst: NoneSet[_PatternFillPropertiesPrst]
    preset: Alias
    fgClr: Typed[ColorChoice, Literal[True]]
    foreground: Alias
    bgClr: Typed[ColorChoice, Literal[True]]
    background: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        prst: _PatternFillPropertiesPrst | Literal["none"] | None = None,
        fgClr: ColorChoice | None = None,
        bgClr: ColorChoice | None = None,
    ) -> None: ...

class RelativeRect(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    l: Incomplete
    left: Alias
    t: Incomplete
    top: Alias
    r: Incomplete
    right: Alias
    b: Incomplete
    bottom: Alias
    def __init__(self, l=None, t=None, r=None, b=None) -> None: ...

class StretchInfoProperties(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    fillRect: Typed[RelativeRect, Literal[True]]
    def __init__(self, fillRect: RelativeRect = ...) -> None: ...

class GradientStop(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    pos: MinMax[float, Literal[True]]
    scrgbClr: Typed[_RGBPercent, Literal[True]]
    RGBPercent: Alias
    srgbClr: NestedValue[_RGBPercent, Literal[True]]
    RGB: Alias
    hslClr: Typed[HSLColor, Literal[True]]
    sysClr: Typed[SystemColor, Literal[True]]
    schemeClr: Typed[SchemeColor, Literal[True]]
    prstClr: NestedNoneSet[_PresetColors]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        pos: ConvertibleToFloat | None = None,
        scrgbClr: _RGBPercent | None = None,
        srgbClr: _HasTagAndGet[_RGBPercent | None] | _RGBPercent | None = None,
        hslClr: HSLColor | None = None,
        sysClr: SystemColor | None = None,
        schemeClr: SchemeColor | None = None,
        prstClr: _NestedNoneSetParam[_PresetColors] = None,
    ) -> None: ...

class LinearShadeProperties(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    ang: Integer[Literal[False]]
    scaled: Bool[Literal[True]]
    def __init__(self, ang: ConvertibleToInt, scaled: _ConvertibleToBool | None = None) -> None: ...

class PathShadeProperties(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    path: Set[_PathShadePropertiesPath]
    fillToRect: Typed[RelativeRect, Literal[True]]
    def __init__(self, path: _PathShadePropertiesPath, fillToRect: RelativeRect | None = None) -> None: ...

class GradientFillProperties(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    flip: NoneSet[_PropertiesFlip]
    rotWithShape: Bool[Literal[True]]
    gsLst: Incomplete
    stop_list: Alias
    lin: Typed[LinearShadeProperties, Literal[True]]
    linear: Alias
    path: Typed[PathShadeProperties, Literal[True]]
    tileRect: Typed[RelativeRect, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        flip: _PropertiesFlip | Literal["none"] | None = None,
        rotWithShape: _ConvertibleToBool | None = None,
        gsLst=(),
        lin: LinearShadeProperties | None = None,
        path: PathShadeProperties | None = None,
        tileRect: RelativeRect | None = None,
    ) -> None: ...

class SolidColorFillProperties(Serialisable):
    tagname: ClassVar[str]
    scrgbClr: Typed[_RGBPercent, Literal[True]]
    RGBPercent: Alias
    srgbClr: NestedValue[_RGBPercent, Literal[True]]
    RGB: Alias
    hslClr: Typed[HSLColor, Literal[True]]
    sysClr: Typed[SystemColor, Literal[True]]
    schemeClr: Typed[SchemeColor, Literal[True]]
    prstClr: NestedNoneSet[_PresetColors]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        scrgbClr: _RGBPercent | None = None,
        srgbClr: _HasTagAndGet[_RGBPercent | None] | _RGBPercent | None = None,
        hslClr: HSLColor | None = None,
        sysClr: SystemColor | None = None,
        schemeClr: SchemeColor | None = None,
        prstClr: _NestedNoneSetParam[_PresetColors] = None,
    ) -> None: ...

class Blip(Serialisable):
    tagname: ClassVar[str]
    namespace: ClassVar[str]
    cstate: NoneSet[_BlipCstate]
    embed: Incomplete
    link: Incomplete
    noGrp: Bool[Literal[True]]
    noSelect: Bool[Literal[True]]
    noRot: Bool[Literal[True]]
    noChangeAspect: Bool[Literal[True]]
    noMove: Bool[Literal[True]]
    noResize: Bool[Literal[True]]
    noEditPoints: Bool[Literal[True]]
    noAdjustHandles: Bool[Literal[True]]
    noChangeArrowheads: Bool[Literal[True]]
    noChangeShapeType: Bool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    alphaBiLevel: Typed[AlphaBiLevelEffect, Literal[True]]
    alphaCeiling: Typed[AlphaCeilingEffect, Literal[True]]
    alphaFloor: Typed[AlphaFloorEffect, Literal[True]]
    alphaInv: Typed[AlphaInverseEffect, Literal[True]]
    alphaMod: Typed[AlphaModulateEffect, Literal[True]]
    alphaModFix: Typed[AlphaModulateFixedEffect, Literal[True]]
    alphaRepl: Typed[AlphaReplaceEffect, Literal[True]]
    biLevel: Typed[BiLevelEffect, Literal[True]]
    blur: Typed[BlurEffect, Literal[True]]
    clrChange: Typed[ColorChangeEffect, Literal[True]]
    clrRepl: Typed[ColorReplaceEffect, Literal[True]]
    duotone: Typed[DuotoneEffect, Literal[True]]
    fillOverlay: Typed[FillOverlayEffect, Literal[True]]
    grayscl: Typed[GrayscaleEffect, Literal[True]]
    hsl: Typed[HSLEffect, Literal[True]]
    lum: Typed[LuminanceEffect, Literal[True]]
    tint: Typed[TintEffect, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        cstate: _BlipCstate | Literal["none"] | None = None,
        embed=None,
        link=None,
        noGrp: _ConvertibleToBool | None = None,
        noSelect: _ConvertibleToBool | None = None,
        noRot: _ConvertibleToBool | None = None,
        noChangeAspect: _ConvertibleToBool | None = None,
        noMove: _ConvertibleToBool | None = None,
        noResize: _ConvertibleToBool | None = None,
        noEditPoints: _ConvertibleToBool | None = None,
        noAdjustHandles: _ConvertibleToBool | None = None,
        noChangeArrowheads: _ConvertibleToBool | None = None,
        noChangeShapeType: _ConvertibleToBool | None = None,
        extLst: ExtensionList | None = None,
        alphaBiLevel: AlphaBiLevelEffect | None = None,
        alphaCeiling: AlphaCeilingEffect | None = None,
        alphaFloor: AlphaFloorEffect | None = None,
        alphaInv: AlphaInverseEffect | None = None,
        alphaMod: AlphaModulateEffect | None = None,
        alphaModFix: AlphaModulateFixedEffect | None = None,
        alphaRepl: AlphaReplaceEffect | None = None,
        biLevel: BiLevelEffect | None = None,
        blur: BlurEffect | None = None,
        clrChange: ColorChangeEffect | None = None,
        clrRepl: ColorReplaceEffect | None = None,
        duotone: DuotoneEffect | None = None,
        fillOverlay: FillOverlayEffect | None = None,
        grayscl: GrayscaleEffect | None = None,
        hsl: HSLEffect | None = None,
        lum: LuminanceEffect | None = None,
        tint: TintEffect | None = None,
    ) -> None: ...

class TileInfoProperties(Serialisable):
    tx: Integer[Literal[True]]
    ty: Integer[Literal[True]]
    sx: Integer[Literal[True]]
    sy: Integer[Literal[True]]
    flip: NoneSet[_PropertiesFlip]
    algn: Set[_TileInfoPropertiesAlgn]
    def __init__(
        self,
        tx: ConvertibleToInt | None = None,
        ty: ConvertibleToInt | None = None,
        sx: ConvertibleToInt | None = None,
        sy: ConvertibleToInt | None = None,
        flip: _PropertiesFlip | Literal["none"] | None = None,
        *,
        algn: _TileInfoPropertiesAlgn,
    ) -> None: ...

class BlipFillProperties(Serialisable):
    tagname: ClassVar[str]
    dpi: Integer[Literal[True]]
    rotWithShape: Bool[Literal[True]]
    blip: Typed[Blip, Literal[True]]
    srcRect: Typed[RelativeRect, Literal[True]]
    tile: Typed[TileInfoProperties, Literal[True]]
    stretch: Typed[StretchInfoProperties, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        dpi: ConvertibleToInt | None = None,
        rotWithShape: _ConvertibleToBool | None = None,
        blip: Blip | None = None,
        tile: TileInfoProperties | None = None,
        stretch: StretchInfoProperties = ...,
        srcRect: RelativeRect | None = None,
    ) -> None: ...



import sys
from _typeshed import FileDescriptor
from collections.abc import Callable
from typing_extensions import ParamSpec, TypeAlias

from gevent._ffi import watcher as _base
from gevent.libev.corecffi import loop as cffi_loop

__all__: list[str] = []

if sys.platform != "win32":
    from gevent.libev.corecext import loop as cext_loop

    _Loop: TypeAlias = cffi_loop | cext_loop
else:
    _Loop: TypeAlias = cffi_loop

_P = ParamSpec("_P")

class watcher(_base.watcher):
    def __init__(self, _loop: _Loop, ref: bool = True, priority: int | None = None) -> None: ...
    @property
    def ref(self) -> bool: ...
    @ref.setter
    def ref(self, value: bool) -> None: ...
    # does not accept keyword arguments
    def feed(self, revents: int, callback: Callable[_P, object], *args: _P.args, **_: _P.kwargs) -> None: ...

class io(_base.IoMixin, watcher):
    EVENT_MASK: int
    @property
    def fd(self) -> FileDescriptor: ...
    @fd.setter
    def fd(self, value: FileDescriptor) -> None: ...
    @property
    def events(self) -> int: ...
    @events.setter
    def events(self, events: int) -> None: ...
    @property
    def events_str(self) -> str: ...

class timer(_base.TimerMixin, watcher):
    @property
    def at(self) -> float: ...

class signal(_base.SignalMixin, watcher): ...
class idle(_base.IdleMixin, watcher): ...
class prepare(_base.PrepareMixin, watcher): ...
class check(_base.CheckMixin, watcher): ...
class fork(_base.ForkMixin, watcher): ...
class async_(_base.AsyncMixin, watcher): ...

class child(_base.ChildMixin, watcher):
    @property
    def rpid(self) -> int: ...
    @rpid.setter
    def rpid(self, value: int) -> None: ...
    @property
    def rstatus(self) -> int: ...
    @rstatus.setter
    def rstatus(self, value: int) -> None: ...

class stat(_base.StatMixin, watcher):
    @property
    def interval(self) -> float: ...



from typing import overload
from typing_extensions import Self

class Angle:
    @overload
    def __init__(self, *, ra: bool = False) -> None: ...
    @overload
    def __init__(self, a: Angle, /, *, ra: bool = False) -> None: ...
    @overload
    def __init__(self, a: float, /, *, ra: bool = False, radians: bool = False) -> None: ...
    @overload
    def __init__(self, a: list[float] | tuple[float, ...], /, *, ra: bool = False, radians: bool = False) -> None: ...
    @overload
    def __init__(self, a1: float, a2: float, /, *, ra: bool = False) -> None: ...
    @overload
    def __init__(self, a1: float, a2: float, a3: float, /, *, ra: bool = False) -> None: ...
    @overload
    def __init__(self, a1: float, a2: float, a3: float, a4: float, /, *, ra: bool = False) -> None: ...
    @staticmethod
    def reduce_deg(deg: float | Angle) -> float: ...
    @staticmethod
    def reduce_dms(degrees: float, minutes: float, seconds: float = 0.0) -> tuple[int, int, float, float]: ...
    @staticmethod
    def deg2dms(deg: float | Angle) -> tuple[int, int, float, float]: ...
    @staticmethod
    def dms2deg(degrees: float, minutes: float, seconds: float = 0.0) -> float: ...
    def get_tolerance(self) -> float: ...
    def set_tolerance(self, tol: float) -> None: ...
    def __call__(self) -> float: ...
    @overload
    def set(self, *, ra: bool = False) -> None: ...
    @overload
    def set(self, a: Angle, /, *, ra: bool = False) -> None: ...
    @overload
    def set(self, a: float, /, *, ra: bool = False, radians: bool = False) -> None: ...
    @overload
    def set(self, a: list[float] | tuple[float, ...], /, *, ra: bool = False, radians: bool = False) -> None: ...
    @overload
    def set(self, a1: float, a2: float, /, *, ra: bool = False) -> None: ...
    @overload
    def set(self, a1: float, a2: float, a3: float, /, *, ra: bool = False) -> None: ...
    @overload
    def set(self, a1: float, a2: float, a3: float, a4: float, /, *, ra: bool = False) -> None: ...
    def set_radians(self, rads: float) -> None: ...
    @overload
    def set_ra(self) -> None: ...
    @overload
    def set_ra(self, a: float | Angle | list[float] | tuple[float, ...], /) -> None: ...
    @overload
    def set_ra(self, a1: float, a2: float, /) -> None: ...
    @overload
    def set_ra(self, a1: float, a2: float, a3: float, /) -> None: ...
    @overload
    def set_ra(self, a1: float, a2: float, a3: float, a4: float, /) -> None: ...
    def dms_str(self, fancy: bool | None = True, n_dec: int = -1) -> str: ...
    def get_ra(self) -> float: ...
    def ra_str(self, fancy: bool | None = True, n_dec: int = -1) -> str: ...
    def rad(self) -> float: ...
    def dms_tuple(self) -> tuple[int, int, float, float]: ...
    def ra_tuple(self) -> tuple[int, int, float, float]: ...
    def to_positive(self) -> Self: ...
    def __eq__(self, b: float | Angle) -> bool: ...  # type: ignore[override]
    def __ne__(self, b: float | Angle) -> bool: ...  # type: ignore[override]
    def __lt__(self, b: float | Angle) -> bool: ...
    def __ge__(self, b: float | Angle) -> bool: ...
    def __gt__(self, b: float | Angle) -> bool: ...
    def __le__(self, b: float | Angle) -> bool: ...
    def __neg__(self) -> Angle: ...
    def __abs__(self) -> Angle: ...
    def __mod__(self, b: float | Angle) -> Angle: ...
    def __add__(self, b: float | Angle) -> Angle: ...
    def __sub__(self, b: float | Angle) -> Angle: ...
    def __mul__(self, b: float | Angle) -> Angle: ...
    def __div__(self, b: float | Angle) -> Angle: ...
    def __truediv__(self, b: float | Angle) -> Angle: ...
    def __pow__(self, b: float | Angle) -> Angle: ...
    def __imod__(self, b: float | Angle) -> Self: ...
    def __iadd__(self, b: float | Angle) -> Self: ...
    def __isub__(self, b: float | Angle) -> Self: ...
    def __imul__(self, b: float | Angle) -> Self: ...
    def __idiv__(self, b: float | Angle) -> Angle: ...
    def __itruediv__(self, b: float | Angle) -> Self: ...
    def __ipow__(self, b: float | Angle) -> Self: ...
    def __rmod__(self, b: float | Angle) -> Angle: ...
    def __radd__(self, b: float | Angle) -> Angle: ...
    def __rsub__(self, b: float | Angle) -> Angle: ...
    def __rmul__(self, b: float | Angle) -> Angle: ...
    def __rdiv__(self, b: float | Angle) -> Angle: ...
    def __rtruediv__(self, b: float | Angle) -> Angle: ...
    def __rpow__(self, b: float | Angle) -> Angle: ...
    def __float__(self) -> float: ...
    def __int__(self) -> int: ...
    def __round__(self, n: float = 0) -> Angle: ...

def main() -> None: ...



from collections.abc import Iterable
from typing import TypeVar, overload
from typing_extensions import TypeAlias

from tensorflow import IndexedSlices, RaggedTensor, Tensor
from tensorflow._aliases import DTypeLike, ShapeLike, TensorCompatible
from tensorflow.sparse import SparseTensor

_TensorCompatibleT = TypeVar("_TensorCompatibleT", bound=TensorCompatible)
_SparseTensorCompatible: TypeAlias = TensorCompatible | SparseTensor

# Most operations support RaggedTensor. Documentation for them is here,
# https://www.tensorflow.org/api_docs/python/tf/ragged.
# Most operations do not support SparseTensor. Operations often don't document
# whether they support SparseTensor and it is best to test them manually. Typically
# if an operation outputs non-zero value for a zero input, it will not support
# SparseTensors. Binary operations with ragged tensors usually only work
# if both operands are ragged.
@overload
def abs(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def abs(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def abs(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def angle(input: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def angle(input: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def sin(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sin(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def cos(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def cos(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def exp(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def exp(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def sinh(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sinh(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def cosh(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def cosh(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def tanh(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def tanh(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def tanh(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def expm1(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def expm1(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def log(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def log(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def log1p(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def log1p(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def negative(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def negative(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def negative(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def sigmoid(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sigmoid(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def add(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def add(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def add_n(inputs: Iterable[TensorCompatible | IndexedSlices], name: str | None = None) -> Tensor: ...
@overload
def add_n(inputs: Iterable[RaggedTensor], name: str | None = None) -> RaggedTensor: ...
@overload
def subtract(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def subtract(x: TensorCompatible | RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def subtract(
    x: TensorCompatible | RaggedTensor, y: TensorCompatible | RaggedTensor, name: str | None = None
) -> Tensor | RaggedTensor: ...
@overload
def multiply(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def multiply(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def multiply_no_nan(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def multiply_no_nan(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def divide(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def divide(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def divide_no_nan(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def divide_no_nan(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def floormod(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def floormod(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def ceil(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def ceil(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def floor(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def floor(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...

# Uses isinstance on list/tuple so other Sequence types are not supported. The TypeVar is to
# behave covariantly.
def accumulate_n(
    inputs: list[_TensorCompatibleT] | tuple[_TensorCompatibleT, ...],
    shape: ShapeLike | None = None,
    tensor_dtype: DTypeLike | None = None,
    name: str | None = None,
) -> Tensor: ...
@overload
def pow(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def pow(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def reciprocal(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def reciprocal(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def is_nan(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def is_nan(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def minimum(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def minimum(x: RaggedTensor, y: TensorCompatible | RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def minimum(x: TensorCompatible | RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def maximum(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def maximum(x: RaggedTensor, y: TensorCompatible | RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def maximum(x: TensorCompatible | RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def logical_not(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def logical_not(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def logical_and(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def logical_and(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def logical_or(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def logical_or(x: RaggedTensor, y: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def logical_xor(x: TensorCompatible, y: TensorCompatible, name: str | None = "LogicalXor") -> Tensor: ...
@overload
def logical_xor(x: RaggedTensor, y: RaggedTensor, name: str | None = "LogicalXor") -> RaggedTensor: ...
@overload
def equal(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def equal(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
@overload
def not_equal(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def not_equal(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
@overload
def greater(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def greater(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
@overload
def greater_equal(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def greater_equal(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
@overload
def less(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def less(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
@overload
def less_equal(x: TensorCompatible, y: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def less_equal(x: RaggedTensor, y: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
def segment_sum(data: TensorCompatible, segment_ids: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sign(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sign(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def sign(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def sqrt(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def sqrt(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def sqrt(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def rsqrt(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def rsqrt(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def square(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def square(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def square(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def softplus(features: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def softplus(features: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def round(x: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def round(x: SparseTensor, name: str | None = None) -> SparseTensor: ...
@overload
def round(x: RaggedTensor, name: str | None = None) -> RaggedTensor: ...

# Depending on the method axis is either a rank 0 tensor or a rank 0/1 tensor.
def reduce_mean(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_sum(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_max(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_min(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_prod(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_std(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_variance(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def argmax(
    input: TensorCompatible, axis: TensorCompatible | None = None, output_type: DTypeLike = ..., name: str | None = None
) -> Tensor: ...
def argmin(
    input: TensorCompatible, axis: TensorCompatible | None = None, output_type: DTypeLike = ..., name: str | None = None
) -> Tensor: ...

# Only for bool tensors.
def reduce_any(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def reduce_all(
    input_tensor: TensorCompatible | RaggedTensor,
    axis: TensorCompatible | None = None,
    keepdims: bool = False,
    name: str | None = None,
) -> Tensor: ...
def count_nonzero(
    input: _SparseTensorCompatible,
    axis: TensorCompatible | None = None,
    keepdims: bool | None = None,
    dtype: DTypeLike = ...,
    name: str | None = None,
) -> Tensor: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_eulerian", "eulerian_circuit", "eulerize", "is_semieulerian", "has_eulerian_path", "eulerian_path"]

@_dispatchable
def is_eulerian(G: Graph[_Node]) -> bool: ...
@_dispatchable
def is_semieulerian(G: Graph[_Node]) -> bool: ...
@_dispatchable
def eulerian_circuit(
    G: Graph[_Node], source: _Node | None = None, keys: bool = False
) -> Generator[Incomplete, Incomplete, None]: ...
@_dispatchable
def has_eulerian_path(G: Graph[_Node], source: _Node | None = None) -> bool: ...
@_dispatchable
def eulerian_path(G: Graph[_Node], source=None, keys: bool = False) -> Generator[Incomplete, Incomplete, None]: ...
@_dispatchable
def eulerize(G: Graph[_Node]): ...



from abc import abstractmethod
from typing import IO, Any, ClassVar

class BaseStorage:
    name: str | None
    read_mode: str
    encoding: str | None
    def __init__(self, *, name: str | None = None, read_mode: str = "", encoding: str | None = None) -> None: ...
    @abstractmethod
    def save(self, data: Any) -> None: ...
    @abstractmethod
    def read(self) -> Any: ...  # `Any` because `read` returns things from `save`
    @abstractmethod
    def remove(self) -> None: ...

class TempFolderStorage(BaseStorage):
    def save(self, data: Any) -> None: ...
    def read(self) -> Any: ...
    def remove(self) -> None: ...
    def get_full_path(self) -> str: ...

class CacheStorage(BaseStorage):
    CACHE_LIFETIME: int
    CACHE_PREFIX: str
    def save(self, data: Any) -> None: ...
    def read(self) -> Any: ...
    def remove(self) -> None: ...

class MediaStorage(BaseStorage):
    MEDIA_FOLDER: ClassVar[str]
    def save(self, data: IO[Any]) -> None: ...
    def read(self) -> Any: ...
    def remove(self) -> None: ...
    def get_full_path(self) -> str: ...



from __future__ import annotations

import sys
import tkinter
import traceback
import types
from typing import Optional, Tuple
from typing_extensions import assert_type


def custom_handler(exc: type[BaseException], val: BaseException, tb: types.TracebackType | None) -> None:
    print("oh no")


root = tkinter.Tk()
root.report_callback_exception = traceback.print_exception
root.report_callback_exception = custom_handler


def foo(x: int, y: str) -> None:
    pass


root.after(1000, foo, 10, "lol")
root.after(1000, foo, 10, 10)  # type: ignore


# Font size must be integer
label = tkinter.Label()
label.config(font=("", 12))
label.config(font=("", 12.34))  # type: ignore
label.config(font=("", 12, "bold"))
label.config(font=("", 12.34, "bold"))  # type: ignore


# Test the `.count()` method. Comments show values that are returned at runtime.
t = tkinter.Text()
t.insert("end", "asd asd asd\nasd asd")

assert_type(t.count("1.0", "2.3"), Optional[Tuple[int]])  # (15,)
assert_type(t.count("2.3", "2.3"), Optional[Tuple[int]])  # None
assert_type(t.count("1.0", "2.3", "indices"), Optional[Tuple[int]])  # (15,)
assert_type(t.count("2.3", "2.3", "indices"), Optional[Tuple[int]])  # None
assert_type(t.count("1.0", "2.3", "indices", "update"), Optional[int])  # 15
assert_type(t.count("2.3", "2.3", "indices", "update"), Optional[int])  # None
assert_type(t.count("1.0", "2.3", "indices", "lines"), Tuple[int, int])  # (15, 1)
assert_type(t.count("2.3", "2.3", "indices", "lines"), Tuple[int, int])  # (0, 0)
assert_type(t.count("1.0", "2.3", "indices", "lines", "update"), Tuple[int, ...])  # (15, 1)
assert_type(t.count("2.3", "2.3", "indices", "lines", "update"), Tuple[int, ...])  # (0, 0)
assert_type(t.count("1.0", "2.3", "indices", "lines", "chars"), Tuple[int, ...])  # (15, 1, 15)
assert_type(t.count("2.3", "2.3", "indices", "lines", "chars"), Tuple[int, ...])  # (0, 0, 0)
assert_type(t.count("1.0", "2.3", "indices", "lines", "chars", "update"), Tuple[int, ...])  # (15, 1, 15)
assert_type(t.count("2.3", "2.3", "indices", "lines", "chars", "update"), Tuple[int, ...])  # (0, 0, 0)
assert_type(t.count("1.0", "2.3", "indices", "lines", "chars", "ypixels"), Tuple[int, ...])  # (15, 1, 15, 19)
assert_type(t.count("2.3", "2.3", "indices", "lines", "chars", "ypixels"), Tuple[int, ...])  # (0, 0, 0, 0)

if sys.version_info >= (3, 13):
    assert_type(t.count("1.0", "2.3", return_ints=True), int)  # 15
    assert_type(t.count("2.3", "2.3", return_ints=True), int)  # 0
    assert_type(t.count("1.0", "2.3", "indices", return_ints=True), int)  # 15
    assert_type(t.count("2.3", "2.3", "indices", return_ints=True), int)  # 0
    assert_type(t.count("1.0", "2.3", "indices", "update", return_ints=True), int)  # 15
    assert_type(t.count("2.3", "2.3", "indices", "update", return_ints=True), int)  # 0
    assert_type(t.count("1.0", "2.3", "indices", "lines", return_ints=True), Tuple[int, int])  # (15, 1)
    assert_type(t.count("2.3", "2.3", "indices", "lines", return_ints=True), Tuple[int, int])  # (0, 0)
    assert_type(t.count("1.0", "2.3", "indices", "lines", "update", return_ints=True), Tuple[int, ...])  # (15, 1)
    assert_type(t.count("2.3", "2.3", "indices", "lines", "update", return_ints=True), Tuple[int, ...])  # (0, 0)
    assert_type(t.count("1.0", "2.3", "indices", "lines", "chars", return_ints=True), Tuple[int, ...])  # (15, 1, 15)
    assert_type(t.count("2.3", "2.3", "indices", "lines", "chars", return_ints=True), Tuple[int, ...])  # (0, 0, 0)
    assert_type(t.count("1.0", "2.3", "indices", "lines", "chars", "update", return_ints=True), Tuple[int, ...])  # (15, 1, 15)
    assert_type(t.count("2.3", "2.3", "indices", "lines", "chars", "update", return_ints=True), Tuple[int, ...])  # (0, 0, 0)
    assert_type(
        t.count("1.0", "2.3", "indices", "lines", "chars", "ypixels", return_ints=True), Tuple[int, ...]
    )  # (15, 1, 15, 19)
    assert_type(t.count("2.3", "2.3", "indices", "lines", "chars", "ypixels", return_ints=True), Tuple[int, ...])  # (0, 0, 0, 0)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ResourceDtypeAndShape(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "shape", b"shape"]) -> None: ...

global___ResourceDtypeAndShape = ResourceDtypeAndShape

@typing.final
class RemoteTensorHandle(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OP_ID_FIELD_NUMBER: builtins.int
    OUTPUT_NUM_FIELD_NUMBER: builtins.int
    DEVICE_FIELD_NUMBER: builtins.int
    OP_DEVICE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    RESOURCE_DTYPES_AND_SHAPES_FIELD_NUMBER: builtins.int
    op_id: builtins.int
    """The ID of the operation that produced this tensor."""
    output_num: builtins.int
    """The index into the outputs of the operation that produced this tensor."""
    device: builtins.str
    """Device where the tensor is located. Cannot be empty.
    For multi-device functions, it's the default device passed to placer.
    """
    op_device: builtins.str
    """Device of the operation producing this tensor. Can be empty if the
    operation producing this tensor is a multi-device function.
    """
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    """Tensor type."""
    @property
    def resource_dtypes_and_shapes(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResourceDtypeAndShape]:
        """Optional data types and shapes of a remote resource variable."""

    def __init__(
        self,
        *,
        op_id: builtins.int | None = ...,
        output_num: builtins.int | None = ...,
        device: builtins.str | None = ...,
        op_device: builtins.str | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        resource_dtypes_and_shapes: collections.abc.Iterable[global___ResourceDtypeAndShape] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "device",
            b"device",
            "dtype",
            b"dtype",
            "op_device",
            b"op_device",
            "op_id",
            b"op_id",
            "output_num",
            b"output_num",
            "resource_dtypes_and_shapes",
            b"resource_dtypes_and_shapes",
        ],
    ) -> None: ...

global___RemoteTensorHandle = RemoteTensorHandle



from collections.abc import Callable, Collection, Mapping, Sequence
from concurrent.futures.thread import ThreadPoolExecutor
from typing import Any

from ..extractor.common import _InfoDict
from .common import FileDownloader
from .http import HttpFD

class HttpQuietDownloader(HttpFD):
    def to_screen(self, *args: Any, **kargs: Any) -> None: ...  # This method is a no-op.
    to_console_title = to_screen

class FragmentFD(FileDownloader):
    def report_retry_fragment(self, err: str, frag_index: int, count: int, retries: int) -> None: ...
    def report_skip_fragment(self, frag_index: int, err: str | None = None) -> None: ...
    def decrypter(self, info_dict: _InfoDict) -> Callable[[Mapping[str, Any], bytes], bytes]: ...
    def download_and_append_fragments_multiple(
        self,
        *args: tuple[Mapping[str, Any], Collection[Mapping[str, Any]], _InfoDict],
        is_fatal: Callable[[int], bool] = ...,
        pack_func: Callable[[str, int], bytes] = ...,
        finish_func: Callable[[], Any] | None = None,
        tpe: ThreadPoolExecutor | None = None,
        interrupt_trigger: Sequence[bool] = (True,),
    ) -> bool: ...
    def download_and_append_fragments(
        self,
        ctx: Mapping[str, Any],
        fragments: Collection[Mapping[str, Any]],
        info_dict: _InfoDict,
        *,
        is_fatal: Callable[[int], bool] = ...,
        pack_func: Callable[[str, int], bytes] = ...,
        finish_func: Callable[[], object] | None = None,
        tpe: ThreadPoolExecutor | None = None,
        interrupt_trigger: Sequence[bool] = (True,),
    ) -> bool: ...



from typing import Final

from ._greenlet import (
    _C_API as _C_API,
    GreenletExit as GreenletExit,
    error as error,
    getcurrent as getcurrent,
    gettrace as gettrace,
    greenlet as greenlet,
    settrace as settrace,
)

__version__: Final[str]
__all__ = ["__version__", "_C_API", "GreenletExit", "error", "getcurrent", "greenlet", "gettrace", "settrace"]



from __future__ import annotations

from typing import Sequence, Tuple
from typing_extensions import assert_type

ints: Sequence[int] = [1, 2, 3]
strs: Sequence[str] = ["one", "two", "three"]
floats: Sequence[float] = [1.0, 2.0, 3.0]
str_tuples: Sequence[Tuple[str]] = list((x,) for x in strs)

assert_type(zip(ints), zip[Tuple[int]])
assert_type(zip(ints, strs), zip[Tuple[int, str]])
assert_type(zip(ints, strs, floats), zip[Tuple[int, str, float]])
assert_type(zip(strs, ints, floats, ints), zip[Tuple[str, int, float, int]])
assert_type(zip(strs, ints, floats, ints, str_tuples), zip[Tuple[str, int, float, int, Tuple[str]]])



from oauthlib.oauth2.rfc6749.tokens import OAuth2Token

from .base import Client

class MobileApplicationClient(Client):
    response_type: str
    def prepare_request_uri(
        self,
        uri,
        redirect_uri: str | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        state: str | None = None,
        **kwargs,
    ) -> str: ...
    token: OAuth2Token
    def parse_request_uri_response(
        self, uri: str, state: str | None = None, scope: str | set[object] | tuple[object] | list[object] | None = None
    ) -> OAuth2Token: ...



from _typeshed import SupportsWrite
from collections.abc import Generator
from typing import Any, Literal
from typing_extensions import TypeAlias

from . import base

# png.Writer; no types available
_Writer: TypeAlias = Any

class PyPNGImage(base.BaseImage):
    kind: str
    allowed_kinds: tuple[Literal["PNG"]]
    # the new_image and get_image methods accept arbitrary keyword arguments to
    # accommodate subclasses with additional arguments.
    def new_image(self, **kwargs: Any) -> _Writer: ...
    def get_image(self, **kwargs: Any) -> _Writer: ...
    def drawrect(self, row: int, col: int) -> None: ...
    def save(self, stream: SupportsWrite[bytes], kind: str | None = None) -> None: ...
    def rows_iter(self) -> Generator[list[int], Any, None]: ...
    def border_rows_iter(self) -> Generator[list[int], Any, None]: ...

PymagingImage = PyPNGImage



from _typeshed import StrPath
from pathlib import Path
from typing import Any, Final, Generic, TypedDict, TypeVar, type_check_only
from typing_extensions import Required

from docutils import Component, nodes
from docutils.frontend import Values
from docutils.io import Output
from docutils.languages import LanguageImporter

_S = TypeVar("_S")

__docformat__: Final = "reStructuredText"

# It would probably be better to specialize writers for subclasses,
# but this gives us all possible Writer items w/o instance checks
@type_check_only
class _WriterParts(TypedDict, total=False):
    # Parts Provided by All Writers https://docutils.sourceforge.io/docs/api/publisher.html#parts-provided-by-all-writers

    # See Writer.assemble_parts
    whole: Required[str | bytes]
    encoding: Required[str]
    errors: Required[str]
    version: Required[str]

    # Parts Provided by the HTML Writers https://docutils.sourceforge.io/docs/api/publisher.html#parts-provided-by-the-html-writers

    # HTML4 Writer https://docutils.sourceforge.io/docs/api/publisher.html#html4-writer
    # + HTML5 Writer https://docutils.sourceforge.io/docs/api/publisher.html#html5-writer
    body: str
    body_prefix: str
    body_pre_docinfo: str
    body_suffix: str
    docinfo: str
    footer: str
    fragment: str
    head: str
    head_prefix: str
    header: str
    html_body: str
    html_head: str
    html_prolog: str
    html_subtitle: str
    html_title: str
    meta: str
    stylesheet: str
    subtitle: str
    title: str
    # PEP/HTML Writer https://docutils.sourceforge.io/docs/api/publisher.html#pep-html-writer
    # + S5/HTML Writer https://docutils.sourceforge.io/docs/api/publisher.html#s5-html-writer
    pepnum: str

    # Parts Provided by the (Xe)LaTeX Writers https://docutils.sourceforge.io/docs/api/publisher.html#parts-provided-by-the-xe-latex-writers

    # (commenting out those already included)
    abstract: str
    # body: str
    # body_pre_docinfo: str
    dedication: str
    # docinfo: str
    fallbacks: str
    # head_prefix: str
    latex_preamble: str
    pdfsetup: str
    requirements: str
    # stylesheet: str
    # subtitle: str
    # title: str
    titledata: str

class Writer(Component, Generic[_S]):
    parts: _WriterParts
    language: LanguageImporter | None = None
    document: nodes.document | None = None
    destination: Output | None = None
    output: _S | None = None
    def __init__(self) -> None: ...
    def write(self, document: nodes.document, destination: Output) -> str | bytes | None: ...
    def translate(self) -> None: ...
    def assemble_parts(self) -> None: ...

class UnfilteredWriter(Writer[_S]): ...

class DoctreeTranslator(nodes.NodeVisitor):
    settings: Values
    def __init__(self, document: nodes.document) -> None: ...
    def uri2path(self, uri: str, output_path: StrPath | None = None) -> Path: ...

WRITER_ALIASES: Final[dict[str, str]]

def get_writer_class(writer_name: str) -> type[Writer[Any]]: ...



from .common import (
    HEADRequest as HEADRequest,
    PATCHRequest as PATCHRequest,
    PUTRequest as PUTRequest,
    Request as Request,
    RequestDirector as RequestDirector,
    RequestHandler as RequestHandler,
    Response as Response,
)



from _typeshed import Incomplete, SupportsItems
from abc import abstractmethod
from collections.abc import Iterable, Sequence
from typing import Any, Final, Literal, NoReturn, TypedDict, type_check_only
from typing_extensions import Self, TypeAlias, Unpack

from reportlab.lib.colors import Color
from reportlab.lib.validators import NoneOr, Validator
from reportlab.pdfgen.canvas import Canvas
from reportlab.platypus import Flowable
from reportlab.platypus.flowables import _HAlignment, _VAlignment

_IntBool: TypeAlias = Literal[0, 1]
_BoolLike: TypeAlias = _IntBool | bool
_PathOp: TypeAlias = (
    tuple[Literal["moveTo"], float, float]
    | tuple[Literal["lineTo"], float, float]
    | tuple[Literal["curveTo"], float, float, float, float, float, float]
    # close path may either be a tuple or just the string
    | Literal["closePath"]
    | tuple[Literal["closePath"]]
    # fallback for list that is not type safe
    | list[Any]
)

# NOTE: These are derived from _attrMap and can optionally be
#       verified at runtime
@type_check_only
class _GroupKwArgs(TypedDict, total=False):
    transform: tuple[float, float, float, float, float, float] | list[float] | list[int]
    # NOTE: This should be used with care, since it will replace elements
    #       it's mostly useful for circumventing validation logic and
    #       reusing the list, rather than populating a new list
    contents: list[Shape]
    strokeOverprint: _BoolLike
    fillOverprint: _BoolLike
    overprintMask: _BoolLike

@type_check_only
class _DrawingKwArgs(_GroupKwArgs, total=False):
    # TODO: Restrict to supported formats?
    formats: list[str] | tuple[str, ...]
    # NOTE: This looks like an implementation detail, so we may not
    #       want to include this in KwArgs
    canv: Canvas
    background: Shape | UserNode | None
    # NOTE: The runtime validation for alignments is incorrect, so
    #       we assume it is turned off and allow all valid values
    hAlign: _HAlignment
    vAlign: _VAlignment
    renderScale: float
    initialFontName: str | None
    initialFontSize: float | None

@type_check_only
class _LineShapeKwArgs(TypedDict, total=False):
    strokeColor: Color | None
    strokeWidth: float
    strokeLineCap: Literal[0, 1, 2]
    strokeLineJoin: Literal[0, 1, 2]
    strokeMiterLimit: float
    strokeDashArray: Sequence[float] | tuple[float, Sequence[float]]
    strokeOpacity: float | None
    strokeOverprint: _BoolLike
    overprintMask: _BoolLike

@type_check_only
class _PathKwArgs(_LineShapeKwArgs, total=False):
    fillColor: Color | None
    fillOpacity: float
    fillOverprint: _BoolLike

@type_check_only
class _AllPathKwArgs(_PathKwArgs, total=False):
    points: list[float] | None
    operators: list[float] | None
    isClipPath: _BoolLike
    autoclose: Literal["svg", "pdf"] | None
    fillMode: Literal[0, 1]

@type_check_only
class _SolidShapeKwArgs(_PathKwArgs, total=False):
    fillMode: Literal[0, 1]

@type_check_only
class _DefinePathKwArgs(_SolidShapeKwArgs, total=False):
    autoclose: Literal["svg", "pdf"] | None
    bbox: tuple[float, float, float, float] | None

@type_check_only
class _WedgeKwArgs(_SolidShapeKwArgs, total=False):
    radius1: float | None
    yradius1: float | None

@type_check_only
class _StringKwArgs(TypedDict, total=False):
    fontName: str
    fontSize: float
    fillColor: Color | None
    textAnchor: Literal["start", "middle", "end", "numeric"]
    encoding: str
    textRenderMode: Literal[0, 1, 2, 3, 4, 5, 6, 7]

__version__: Final[str]
isOpacity: NoneOr
NON_ZERO_WINDING: Final[str]
EVEN_ODD: Final[str]
STATE_DEFAULTS: Final[Incomplete]

class _DrawTimeResizeable: ...

class _SetKeyWordArgs:
    def __init__(self, keywords: SupportsItems[str, Any] = {}) -> None: ...

def getRectsBounds(rectList): ...
def getPathBounds(points): ...
def getPointsBounds(pointList): ...

class Shape(_SetKeyWordArgs, _DrawTimeResizeable):
    @abstractmethod
    def copy(self) -> Self: ...
    def getProperties(self, recur: int = 1) -> dict[str, Any]: ...
    def setProperties(self, props) -> None: ...
    def dumpProperties(self, prefix: str = "") -> None: ...
    def verify(self) -> None: ...
    @abstractmethod
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Group(Shape):
    contents: list[Shape]
    transform: tuple[float, float, float, float, float, float] | list[float] | list[int]
    def __init__(self, *elements: Shape | UserNode, **keywords: Unpack[_GroupKwArgs]) -> None: ...
    def add(self, node: Shape | UserNode, name: str | None = None) -> None: ...
    def insert(self, i: int, n: Shape | UserNode, name: str | None = None) -> None: ...
    def expandUserNodes(self) -> Group: ...
    def copy(self) -> Self: ...
    def rotate(self, theta: float, cx: float = 0, cy: float = 0) -> None: ...
    def translate(self, dx: float, dy: float = 0) -> None: ...
    def scale(self, sx: float, sy: float = 1) -> None: ...
    def skew(self, kx: float, ky: float = 0) -> None: ...
    def shift(self, x: float, y: float = 0) -> None: ...
    # NOTE: This changes the object to a Drawing, rather than returning
    #       a new one, which is not ideal...
    def asDrawing(self, width: float, height: float) -> None: ...
    def getContents(self) -> list[Shape | UserNode]: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Drawing(Group, Flowable):
    background: Shape | UserNode | None
    renderScale: float
    def __init__(
        self, width: float = 400, height: float = 200, *nodes: Shape | UserNode, **keywords: Unpack[_DrawingKwArgs]
    ) -> None: ...
    def draw(self, showBoundary=...) -> None: ...
    def expandUserNodes(self) -> Drawing: ...
    def asGroup(self, *args: Shape | UserNode, **kw: Unpack[_GroupKwArgs]) -> Group: ...
    def save(
        self,
        formats: Iterable[str] | None = None,
        verbose: bool | None = None,
        fnRoot: str | None = None,
        outDir: str | None = None,
        title: str = "",
        **kw,
    ): ...
    def asString(self, format: str, verbose: bool | None = None, preview: int = 0, **kw) -> str: ...
    def resized(
        self, kind: Literal["fit", "fitx", "fity"] = "fit", lpad: float = 0, rpad: float = 0, bpad: float = 0, tpad: float = 0
    ) -> Drawing: ...

class _DrawingEditorMixin: ...

@type_check_only
class _isStrokeDashArray(Validator):
    def test(self, x): ...

isStrokeDashArray: _isStrokeDashArray

class LineShape(Shape):
    strokeColor: Color | None
    strokeWidth: float
    strokeLineCap: Literal[0, 1, 2]
    strokeLineJoin: Literal[0, 1, 2]
    strokeMiterLimit: float
    strokeDashArray: Sequence[float] | tuple[float, Sequence[float]]
    strokeOpacity: float | None
    def __init__(self, kw: _LineShapeKwArgs) -> None: ...
    @abstractmethod
    def copy(self) -> Self: ...
    @abstractmethod
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Line(LineShape):
    x1: float
    y1: float
    x2: float
    y2: float
    def __init__(self, x1: float, y1: float, x2: float, y2: float, **kw: Unpack[_LineShapeKwArgs]) -> None: ...
    # NOTE: For some reason Line doesn't implement copy
    def copy(self) -> NoReturn: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class SolidShape(LineShape):
    fillColor: Color | None
    fillOpacity: float | None
    def __init__(self, kw: _SolidShapeKwArgs) -> None: ...
    @abstractmethod
    def copy(self) -> Self: ...
    @abstractmethod
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Path(SolidShape):
    points: list[float]
    operators: list[float]
    isClipPath: _BoolLike
    autoclose: Literal["svg", "pdf"] | None
    fillMode: Literal[0, 1]
    def __init__(
        self,
        points: list[float] | None = None,
        operators: list[float] | None = None,
        isClipPath: _BoolLike = 0,
        autoclose: Literal["svg", "pdf"] | None = None,
        fillMode: Literal[0, 1] = 0,
        **kw: Unpack[_PathKwArgs],
    ) -> None: ...
    def copy(self) -> Self: ...
    def moveTo(self, x: float, y: float) -> None: ...
    def lineTo(self, x: float, y: float) -> None: ...
    def curveTo(self, x1: float, y1: float, x2: float, y2: float, x3: float, y3: float) -> None: ...
    def closePath(self) -> None: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

EmptyClipPath: Final[Path]

def getArcPoints(
    centerx: float,
    centery: float,
    radius: float,
    startangledegrees: float,
    endangledegrees: float,
    yradius: float | None = None,
    degreedelta: float | None = None,
    reverse: _BoolLike | None = None,
) -> list[float]: ...

class ArcPath(Path):
    def addArc(
        self,
        centerx: float,
        centery: float,
        radius: float,
        startangledegrees: float,
        endangledegrees: float,
        yradius: float | None = None,
        degreedelta: float | None = None,
        moveTo: _BoolLike | None = None,
        reverse: _BoolLike | None = None,
    ) -> None: ...

def definePath(
    pathSegs: Iterable[_PathOp] = [], isClipPath: _BoolLike = 0, dx: float = 0, dy: float = 0, **kw: Unpack[_DefinePathKwArgs]
) -> Path: ...

class Rect(SolidShape):
    x: float
    y: float
    width: float
    height: float
    rx: float
    ry: float
    def __init__(
        self, x: float, y: float, width: float, height: float, rx: float = 0, ry: float = 0, **kw: _SolidShapeKwArgs
    ) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Image(SolidShape):
    x: float
    y: float
    width: float
    height: float
    path: Incomplete
    def __init__(self, x: float, y: float, width: float, height: float, path, **kw: Unpack[_SolidShapeKwArgs]) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Circle(SolidShape):
    cx: float
    cy: float
    r: float
    def __init__(self, cx: float, cy: float, r: float, **kw: Unpack[_SolidShapeKwArgs]) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Ellipse(SolidShape):
    cx: float
    cy: float
    rx: float
    ry: float
    def __init__(self, cx: float, cy: float, rx: float, ry: float, **kw: Unpack[_SolidShapeKwArgs]) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Wedge(SolidShape):
    centerx: float
    centery: float
    radius: float
    startangledegrees: float
    endangledegrees: float
    yradius: float | None
    annular: bool
    # NOTE: This one is not actually settable on the instance if runtime validation
    #       is turned on, but it seems bad to disallow it anyways
    degreedelta: float
    def __init__(
        self,
        centerx: float,
        centery: float,
        radius: float,
        startangledegrees: float,
        endangledegrees: float,
        yradius: float | None = None,
        annular: bool = False,
        **kw: Unpack[_WedgeKwArgs],
    ) -> None: ...
    def asPolygon(self) -> Path | Polygon: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Polygon(SolidShape):
    points: list[float]
    def __init__(self, points: list[float] = [], **kw: Unpack[_SolidShapeKwArgs]) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class PolyLine(LineShape):
    points: list[float]
    def __init__(self, points: list[float] = [], **kw: Unpack[_SolidShapeKwArgs]) -> None: ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class Hatching(Path):
    xyLists: Sequence[tuple[float, float]]
    angles: Sequence[float]
    spacings: Sequence[float]
    def __init__(
        self,
        spacings: float | Sequence[float] = 2,
        angles: float | Sequence[float] = 45,
        xyLists: Sequence[tuple[float, float] | list[float]] = [],
        **kwds: Unpack[_AllPathKwArgs],
    ) -> None: ...

def numericXShift(
    tA, text: str, w: float, fontName: str, fontSize: float, encoding: str | None = None, pivotCharacter: str = "."
) -> float: ...

class String(Shape):
    encoding: str
    x: float
    y: float
    text: str
    textAnchor: Literal["start", "middle", "end", "numeric"]
    fontName: str
    fontSize: float
    fillColor: Color | None
    def __init__(self, x: float, y: float, text: str, **kw: Unpack[_StringKwArgs]) -> None: ...
    def getEast(self): ...
    def copy(self) -> Self: ...
    def getBounds(self) -> tuple[float, float, float, float]: ...

class UserNode(_DrawTimeResizeable):
    @abstractmethod
    def provideNode(self) -> Shape: ...

class DirectDraw(Shape):
    @abstractmethod
    def drawDirectly(self, canvas: Canvas) -> None: ...

def test() -> None: ...



############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import asyncio
import functools
import inspect
import itertools
import logging
from typing import Any, Callable, Dict, Optional, get_type_hints

from pygls.constants import (
    ATTR_COMMAND_TYPE,
    ATTR_EXECUTE_IN_THREAD,
    ATTR_FEATURE_TYPE,
    ATTR_REGISTERED_NAME,
    ATTR_REGISTERED_TYPE,
    PARAM_LS,
)
from pygls.exceptions import (
    CommandAlreadyRegisteredError,
    FeatureAlreadyRegisteredError,
    ThreadDecoratorError,
    ValidationError,
)
from pygls.lsp import get_method_options_type, is_instance

logger = logging.getLogger(__name__)


def assign_help_attrs(f, reg_name, reg_type):
    setattr(f, ATTR_REGISTERED_NAME, reg_name)
    setattr(f, ATTR_REGISTERED_TYPE, reg_type)


def assign_thread_attr(f):
    setattr(f, ATTR_EXECUTE_IN_THREAD, True)


def get_help_attrs(f):
    return getattr(f, ATTR_REGISTERED_NAME, None), getattr(
        f, ATTR_REGISTERED_TYPE, None
    )


def has_ls_param_or_annotation(f, annotation):
    """Returns true if callable has first parameter named `ls` or type of
    annotation"""
    try:
        sig = inspect.signature(f)
        first_p = next(itertools.islice(sig.parameters.values(), 0, 1))
        return first_p.name == PARAM_LS or get_type_hints(f)[first_p.name] == annotation
    except Exception:
        return False


def is_thread_function(f):
    return getattr(f, ATTR_EXECUTE_IN_THREAD, False)


def wrap_with_server(f, server):
    """Returns a new callable/coroutine with server as first argument."""
    if not has_ls_param_or_annotation(f, type(server)):
        return f

    if asyncio.iscoroutinefunction(f):

        async def wrapped(*args, **kwargs):
            return await f(server, *args, **kwargs)

    else:
        wrapped = functools.partial(f, server)
        if is_thread_function(f):
            assign_thread_attr(wrapped)

    return wrapped


class FeatureManager:
    """A class for managing server features.

    Attributes:
        _builtin_features(dict): Predefined set of lsp methods
        _feature_options(dict): Registered feature's options
        _features(dict): Registered features
        _commands(dict): Registered commands
        server(LanguageServer): Reference to the language server
                                If passed, server will be passed to registered
                                features/commands with first parameter:
                                    1. ls - parameter naming convention
                                    2. name: LanguageServer - add typings
    """

    def __init__(self, server=None, converter=None):
        self._builtin_features = {}
        self._feature_options = {}
        self._features = {}
        self._commands = {}
        self.server = server
        self.converter = converter

    def add_builtin_feature(self, feature_name: str, func: Callable) -> None:
        """Registers builtin (predefined) feature."""
        self._builtin_features[feature_name] = func
        logger.info("Registered builtin feature %s", feature_name)

    @property
    def builtin_features(self) -> Dict:
        """Returns server builtin features."""
        return self._builtin_features

    def command(self, command_name: str) -> Callable:
        """Decorator used to register custom commands.

        Example:
            @ls.command('myCustomCommand')
        """

        def decorator(f):
            # Validate
            if command_name is None or command_name.strip() == "":
                logger.error("Missing command name.")
                raise ValidationError("Command name is required.")

            # Check if not already registered
            if command_name in self._commands:
                logger.error('Command "%s" is already registered.', command_name)
                raise CommandAlreadyRegisteredError(command_name)

            assign_help_attrs(f, command_name, ATTR_COMMAND_TYPE)

            wrapped = wrap_with_server(f, self.server)
            # Assign help attributes for thread decorator
            assign_help_attrs(wrapped, command_name, ATTR_COMMAND_TYPE)

            self._commands[command_name] = wrapped

            logger.info('Command "%s" is successfully registered.', command_name)

            return f

        return decorator

    @property
    def commands(self) -> Dict:
        """Returns registered custom commands."""
        return self._commands

    def feature(
        self,
        feature_name: str,
        options: Optional[Any] = None,
    ) -> Callable:
        """Decorator used to register LSP features.

        Example:
            @ls.feature('textDocument/completion', CompletionItems(trigger_characters=['.']))
        """

        def decorator(f):
            # Validate
            if feature_name is None or feature_name.strip() == "":
                logger.error("Missing feature name.")
                raise ValidationError("Feature name is required.")

            # Add feature if not exists
            if feature_name in self._features:
                logger.error('Feature "%s" is already registered.', feature_name)
                raise FeatureAlreadyRegisteredError(feature_name)

            assign_help_attrs(f, feature_name, ATTR_FEATURE_TYPE)

            wrapped = wrap_with_server(f, self.server)
            # Assign help attributes for thread decorator
            assign_help_attrs(wrapped, feature_name, ATTR_FEATURE_TYPE)

            self._features[feature_name] = wrapped

            if options:
                options_type = get_method_options_type(feature_name)
                if options_type and not is_instance(
                    self.converter, options, options_type
                ):
                    raise TypeError(
                        (
                            f'Options of method "{feature_name}"'
                            f" should be instance of type {options_type}"
                        )
                    )
                self._feature_options[feature_name] = options

            logger.info('Registered "%s" with options "%s"', feature_name, options)

            return f

        return decorator

    @property
    def feature_options(self) -> Dict:
        """Returns feature options for registered features."""
        return self._feature_options

    @property
    def features(self) -> Dict:
        """Returns registered features"""
        return self._features

    def thread(self) -> Callable:
        """Decorator that mark function to execute it in a thread."""

        def decorator(f):
            if asyncio.iscoroutinefunction(f):
                raise ThreadDecoratorError(
                    f'Thread decorator cannot be used with async functions "{f.__name__}"'
                )

            # Allow any decorator order
            try:
                reg_name = getattr(f, ATTR_REGISTERED_NAME)
                reg_type = getattr(f, ATTR_REGISTERED_TYPE)

                if reg_type is ATTR_FEATURE_TYPE:
                    assign_thread_attr(self.features[reg_name])
                elif reg_type is ATTR_COMMAND_TYPE:
                    assign_thread_attr(self.commands[reg_name])

            except AttributeError:
                assign_thread_attr(f)

            return f

        return decorator



from .compilers.C.errors import (
    CompileError as CompileError,
    Error as _Error,
    LibError as LibError,
    LinkError as LinkError,
    PreprocessError as PreprocessError,
    UnknownFileType as _UnknownFileType,
)

CCompilerError = _Error
UnknownFileError = _UnknownFileType

class DistutilsError(Exception): ...
class DistutilsModuleError(DistutilsError): ...
class DistutilsClassError(DistutilsError): ...
class DistutilsGetoptError(DistutilsError): ...
class DistutilsArgError(DistutilsError): ...
class DistutilsFileError(DistutilsError): ...
class DistutilsOptionError(DistutilsError): ...
class DistutilsSetupError(DistutilsError): ...
class DistutilsPlatformError(DistutilsError): ...
class DistutilsExecError(DistutilsError): ...
class DistutilsInternalError(DistutilsError): ...
class DistutilsTemplateError(DistutilsError): ...
class DistutilsByteCompileError(DistutilsError): ...



from __future__ import annotations

import sys
from typing import TypeVar
from typing_extensions import assert_type

if sys.version_info >= (3, 11):
    # This can be removed later, but right now Flake8 does not know
    # about these two classes:
    from builtins import BaseExceptionGroup, ExceptionGroup

    # BaseExceptionGroup
    # ==================
    # `BaseExceptionGroup` can work with `BaseException`:
    beg = BaseExceptionGroup("x", [SystemExit(), SystemExit()])
    assert_type(beg, BaseExceptionGroup[SystemExit])
    assert_type(beg.exceptions, tuple[SystemExit | BaseExceptionGroup[SystemExit], ...])

    # Covariance works:
    _beg1: BaseExceptionGroup[BaseException] = beg

    # `BaseExceptionGroup` can work with `Exception`:
    beg2 = BaseExceptionGroup("x", [ValueError()])
    # FIXME: this is not right, runtime returns `ExceptionGroup` instance instead,
    # but I am unable to represent this with types right now.
    assert_type(beg2, BaseExceptionGroup[ValueError])

    # .subgroup()
    # -----------

    assert_type(beg.subgroup(KeyboardInterrupt), BaseExceptionGroup[KeyboardInterrupt] | None)
    assert_type(beg.subgroup((KeyboardInterrupt,)), BaseExceptionGroup[KeyboardInterrupt] | None)

    def is_base_exc(exc: BaseException) -> bool:
        return isinstance(exc, BaseException)

    def is_specific(exc: SystemExit | BaseExceptionGroup[SystemExit]) -> bool:
        return isinstance(exc, SystemExit)

    # This one does not have `BaseExceptionGroup` part,
    # this is why we treat as an error.
    def is_system_exit(exc: SystemExit) -> bool:
        return isinstance(exc, SystemExit)

    def unrelated_subgroup(exc: KeyboardInterrupt) -> bool:
        return False

    assert_type(beg.subgroup(is_base_exc), BaseExceptionGroup[SystemExit] | None)
    assert_type(beg.subgroup(is_specific), BaseExceptionGroup[SystemExit] | None)
    beg.subgroup(is_system_exit)  # type: ignore
    beg.subgroup(unrelated_subgroup)  # type: ignore

    # `Exception`` subgroup returns `ExceptionGroup`:
    assert_type(beg.subgroup(ValueError), ExceptionGroup[ValueError] | None)
    assert_type(beg.subgroup((ValueError,)), ExceptionGroup[ValueError] | None)

    # Callable are harder, we don't support cast to `ExceptionGroup` here.
    # Because callables might return `True` the first time. And `BaseExceptionGroup`
    # will stick, no matter what arguments are.

    def is_exception(exc: Exception) -> bool:
        return isinstance(exc, Exception)

    def is_exception_or_beg(exc: Exception | BaseExceptionGroup[SystemExit]) -> bool:
        return isinstance(exc, Exception)

    # This is an error because of the `Exception` argument type,
    # while `SystemExit` is needed instead.
    beg.subgroup(is_exception_or_beg)  # type: ignore

    # This is an error, because `BaseExceptionGroup` is not an `Exception`
    # subclass. It is required.
    beg.subgroup(is_exception)  # type: ignore

    # .split()
    # --------

    assert_type(
        beg.split(KeyboardInterrupt), tuple[BaseExceptionGroup[KeyboardInterrupt] | None, BaseExceptionGroup[SystemExit] | None]
    )
    assert_type(
        beg.split((KeyboardInterrupt,)),
        tuple[BaseExceptionGroup[KeyboardInterrupt] | None, BaseExceptionGroup[SystemExit] | None],
    )
    assert_type(
        beg.split(ValueError),  # there are no `ValueError` items in there, but anyway
        tuple[ExceptionGroup[ValueError] | None, BaseExceptionGroup[SystemExit] | None],
    )

    excs_to_split: list[ValueError | KeyError | SystemExit] = [ValueError(), KeyError(), SystemExit()]
    to_split = BaseExceptionGroup("x", excs_to_split)
    assert_type(to_split, BaseExceptionGroup[ValueError | KeyError | SystemExit])

    # Ideally the first part should be `ExceptionGroup[ValueError]` (done)
    # and the second part should be `BaseExceptionGroup[KeyError | SystemExit]`,
    # but we cannot subtract type from a union.
    # We also cannot change `BaseExceptionGroup` to `ExceptionGroup` even if needed
    # in the second part here because of that.
    assert_type(
        to_split.split(ValueError),
        tuple[ExceptionGroup[ValueError] | None, BaseExceptionGroup[ValueError | KeyError | SystemExit] | None],
    )

    def split_callable1(exc: ValueError | KeyError | SystemExit | BaseExceptionGroup[ValueError | KeyError | SystemExit]) -> bool:
        return True

    assert_type(
        to_split.split(split_callable1),  # Concrete type is ok
        tuple[
            BaseExceptionGroup[ValueError | KeyError | SystemExit] | None,
            BaseExceptionGroup[ValueError | KeyError | SystemExit] | None,
        ],
    )
    assert_type(
        to_split.split(is_base_exc),  # Base class is ok
        tuple[
            BaseExceptionGroup[ValueError | KeyError | SystemExit] | None,
            BaseExceptionGroup[ValueError | KeyError | SystemExit] | None,
        ],
    )
    # `Exception` cannot be used: `BaseExceptionGroup` is not a subtype of it.
    to_split.split(is_exception)  # type: ignore

    # .derive()
    # ---------

    assert_type(beg.derive([ValueError()]), ExceptionGroup[ValueError])
    assert_type(beg.derive([KeyboardInterrupt()]), BaseExceptionGroup[KeyboardInterrupt])

    # ExceptionGroup
    # ==============

    # `ExceptionGroup` can work with `Exception`:
    excs: list[ValueError | KeyError] = [ValueError(), KeyError()]
    eg = ExceptionGroup("x", excs)
    assert_type(eg, ExceptionGroup[ValueError | KeyError])
    assert_type(eg.exceptions, tuple[ValueError | KeyError | ExceptionGroup[ValueError | KeyError], ...])

    # Covariance works:
    _eg1: ExceptionGroup[Exception] = eg

    # `ExceptionGroup` cannot work with `BaseException`:
    ExceptionGroup("x", [SystemExit()])  # type: ignore

    # .subgroup()
    # -----------

    # Our decision is to ban cases like::
    #
    #   >>> eg = ExceptionGroup('x', [ValueError()])
    #   >>> eg.subgroup(BaseException)
    #   ExceptionGroup('e', [ValueError()])
    #
    # are possible in runtime.
    # We do it because, it does not make sense for all other base exception types.
    # Supporting just `BaseException` looks like an overkill.
    eg.subgroup(BaseException)  # type: ignore
    eg.subgroup((KeyboardInterrupt, SystemExit))  # type: ignore

    assert_type(eg.subgroup(Exception), ExceptionGroup[Exception] | None)
    assert_type(eg.subgroup(ValueError), ExceptionGroup[ValueError] | None)
    assert_type(eg.subgroup((ValueError,)), ExceptionGroup[ValueError] | None)

    def subgroup_eg1(exc: ValueError | KeyError | ExceptionGroup[ValueError | KeyError]) -> bool:
        return True

    def subgroup_eg2(exc: ValueError | KeyError) -> bool:
        return True

    assert_type(eg.subgroup(subgroup_eg1), ExceptionGroup[ValueError | KeyError] | None)
    assert_type(eg.subgroup(is_exception), ExceptionGroup[ValueError | KeyError] | None)
    assert_type(eg.subgroup(is_base_exc), ExceptionGroup[ValueError | KeyError] | None)
    assert_type(eg.subgroup(is_base_exc), ExceptionGroup[ValueError | KeyError] | None)

    # Does not have `ExceptionGroup` part:
    eg.subgroup(subgroup_eg2)  # type: ignore

    # .split()
    # --------

    assert_type(eg.split(TypeError), tuple[ExceptionGroup[TypeError] | None, ExceptionGroup[ValueError | KeyError] | None])
    assert_type(eg.split((TypeError,)), tuple[ExceptionGroup[TypeError] | None, ExceptionGroup[ValueError | KeyError] | None])
    assert_type(
        eg.split(is_exception), tuple[ExceptionGroup[ValueError | KeyError] | None, ExceptionGroup[ValueError | KeyError] | None]
    )
    assert_type(
        eg.split(is_base_exc),
        # is not converted, because `ExceptionGroup` cannot have
        # direct `BaseException` subclasses inside.
        tuple[ExceptionGroup[ValueError | KeyError] | None, ExceptionGroup[ValueError | KeyError] | None],
    )

    # It does not include `ExceptionGroup` itself, so it will fail:
    def value_or_key_error(exc: ValueError | KeyError) -> bool:
        return isinstance(exc, (ValueError, KeyError))

    eg.split(value_or_key_error)  # type: ignore

    # `ExceptionGroup` cannot have direct `BaseException` subclasses inside.
    eg.split(BaseException)  # type: ignore
    eg.split((SystemExit, GeneratorExit))  # type: ignore

    # .derive()
    # ---------

    assert_type(eg.derive([ValueError()]), ExceptionGroup[ValueError])
    assert_type(eg.derive([KeyboardInterrupt()]), BaseExceptionGroup[KeyboardInterrupt])

    # BaseExceptionGroup Custom Subclass
    # ==================================
    # In some cases `Self` type can be preserved in runtime,
    # but it is impossible to express. That's why we always fallback to
    # `BaseExceptionGroup` and `ExceptionGroup`.

    _BE = TypeVar("_BE", bound=BaseException)

    class CustomBaseGroup(BaseExceptionGroup[_BE]): ...

    cb1 = CustomBaseGroup("x", [SystemExit()])
    assert_type(cb1, CustomBaseGroup[SystemExit])
    cb2 = CustomBaseGroup("x", [ValueError()])
    assert_type(cb2, CustomBaseGroup[ValueError])

    # .subgroup()
    # -----------

    assert_type(cb1.subgroup(KeyboardInterrupt), BaseExceptionGroup[KeyboardInterrupt] | None)
    assert_type(cb2.subgroup((KeyboardInterrupt,)), BaseExceptionGroup[KeyboardInterrupt] | None)

    assert_type(cb1.subgroup(ValueError), ExceptionGroup[ValueError] | None)
    assert_type(cb2.subgroup((KeyError,)), ExceptionGroup[KeyError] | None)

    def cb_subgroup1(exc: SystemExit | CustomBaseGroup[SystemExit]) -> bool:
        return True

    def cb_subgroup2(exc: ValueError | CustomBaseGroup[ValueError]) -> bool:
        return True

    assert_type(cb1.subgroup(cb_subgroup1), BaseExceptionGroup[SystemExit] | None)
    assert_type(cb2.subgroup(cb_subgroup2), BaseExceptionGroup[ValueError] | None)
    cb1.subgroup(cb_subgroup2)  # type: ignore
    cb2.subgroup(cb_subgroup1)  # type: ignore

    # .split()
    # --------

    assert_type(
        cb1.split(KeyboardInterrupt), tuple[BaseExceptionGroup[KeyboardInterrupt] | None, BaseExceptionGroup[SystemExit] | None]
    )
    assert_type(cb1.split(TypeError), tuple[ExceptionGroup[TypeError] | None, BaseExceptionGroup[SystemExit] | None])
    assert_type(cb2.split((TypeError,)), tuple[ExceptionGroup[TypeError] | None, BaseExceptionGroup[ValueError] | None])

    def cb_split1(exc: SystemExit | CustomBaseGroup[SystemExit]) -> bool:
        return True

    def cb_split2(exc: ValueError | CustomBaseGroup[ValueError]) -> bool:
        return True

    assert_type(cb1.split(cb_split1), tuple[BaseExceptionGroup[SystemExit] | None, BaseExceptionGroup[SystemExit] | None])
    assert_type(cb2.split(cb_split2), tuple[BaseExceptionGroup[ValueError] | None, BaseExceptionGroup[ValueError] | None])
    cb1.split(cb_split2)  # type: ignore
    cb2.split(cb_split1)  # type: ignore

    # .derive()
    # ---------

    # Note, that `Self` type is not preserved in runtime.
    assert_type(cb1.derive([ValueError()]), ExceptionGroup[ValueError])
    assert_type(cb1.derive([KeyboardInterrupt()]), BaseExceptionGroup[KeyboardInterrupt])
    assert_type(cb2.derive([ValueError()]), ExceptionGroup[ValueError])
    assert_type(cb2.derive([KeyboardInterrupt()]), BaseExceptionGroup[KeyboardInterrupt])

    # ExceptionGroup Custom Subclass
    # ==============================

    _E = TypeVar("_E", bound=Exception)

    class CustomGroup(ExceptionGroup[_E]): ...

    CustomGroup("x", [SystemExit()])  # type: ignore
    cg1 = CustomGroup("x", [ValueError()])
    assert_type(cg1, CustomGroup[ValueError])

    # .subgroup()
    # -----------

    cg1.subgroup(BaseException)  # type: ignore
    cg1.subgroup((KeyboardInterrupt, SystemExit))  # type: ignore

    assert_type(cg1.subgroup(ValueError), ExceptionGroup[ValueError] | None)
    assert_type(cg1.subgroup((KeyError,)), ExceptionGroup[KeyError] | None)

    def cg_subgroup1(exc: ValueError | CustomGroup[ValueError]) -> bool:
        return True

    def cg_subgroup2(exc: ValueError) -> bool:
        return True

    assert_type(cg1.subgroup(cg_subgroup1), ExceptionGroup[ValueError] | None)
    cg1.subgroup(cb_subgroup2)  # type: ignore

    # .split()
    # --------

    assert_type(cg1.split(TypeError), tuple[ExceptionGroup[TypeError] | None, ExceptionGroup[ValueError] | None])
    assert_type(cg1.split((TypeError,)), tuple[ExceptionGroup[TypeError] | None, ExceptionGroup[ValueError] | None])
    cg1.split(BaseException)  # type: ignore

    def cg_split1(exc: ValueError | CustomGroup[ValueError]) -> bool:
        return True

    def cg_split2(exc: ValueError) -> bool:
        return True

    assert_type(cg1.split(cg_split1), tuple[ExceptionGroup[ValueError] | None, ExceptionGroup[ValueError] | None])
    cg1.split(cg_split2)  # type: ignore

    # .derive()
    # ---------

    # Note, that `Self` type is not preserved in runtime.
    assert_type(cg1.derive([ValueError()]), ExceptionGroup[ValueError])
    assert_type(cg1.derive([KeyboardInterrupt()]), BaseExceptionGroup[KeyboardInterrupt])

    # Additional tests
    # ==============================

    def test_exception_group_default_type() -> None:
        try:
            ex: ExceptionGroup = ExceptionGroup("", [ValueError("a"), ValueError("b")])
            raise ex
        except ExceptionGroup as e:
            assert all(isinstance(exc, ValueError) for exc in e.exceptions)

    def test_exception_group_with_specific_type() -> None:
        try:
            ex: ExceptionGroup[ValueError] = ExceptionGroup("", [ValueError("a"), ValueError("b")])
            raise ex
        except ExceptionGroup as e:
            assert all(isinstance(exc, ValueError) for exc in e.exceptions)

    def test_base_exception_group_default_type() -> None:
        try:
            ex: BaseExceptionGroup = BaseExceptionGroup("", [SystemExit("a"), SystemExit("b")])
            raise ex
        except BaseExceptionGroup as e:
            assert all(isinstance(exc, SystemExit) for exc in e.exceptions)

    def test_base_exception_group_with_specific_type() -> None:
        try:
            ex: BaseExceptionGroup[SystemExit] = BaseExceptionGroup("", [SystemExit("a"), SystemExit("b")])
            raise ex
        except BaseExceptionGroup as e:
            assert all(isinstance(exc, SystemExit) for exc in e.exceptions)



"""Core constructs for Jac Language - re-exports."""

from jaclang.pycore.archetype import (
    AccessLevel,
    Anchor,
    Archetype,
    EdgeAnchor,
    EdgeArchetype,
    GenericEdge,
    NodeAnchor,
    NodeArchetype,
    ObjectSpatialFunction,
    Root,
    WalkerAnchor,
    WalkerArchetype,
)
from jaclang.pycore.mtp import MTIR, MTRuntime

__all__ = [
    "AccessLevel",
    "Anchor",
    "NodeAnchor",
    "EdgeAnchor",
    "WalkerAnchor",
    "Archetype",
    "NodeArchetype",
    "EdgeArchetype",
    "WalkerArchetype",
    "GenericEdge",
    "Root",
    "MTIR",
    "MTRuntime",
    "ObjectSpatialFunction",
]



from _typeshed import Incomplete
from collections.abc import Callable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

from .preflowpush import preflow_push

__all__ = ["maximum_flow", "maximum_flow_value", "minimum_cut", "minimum_cut_value"]
default_flow_func = preflow_push

@_dispatchable
def maximum_flow(
    flowG: Graph[_Node],
    _s: _Node,
    _t: _Node,
    capacity: str = "capacity",
    flow_func: Callable[..., Incomplete] | None = None,
    **kwargs,
): ...
@_dispatchable
def maximum_flow_value(
    flowG: Graph[_Node],
    _s: _Node,
    _t: _Node,
    capacity: str = "capacity",
    flow_func: Callable[..., Incomplete] | None = None,
    **kwargs,
): ...
@_dispatchable
def minimum_cut(
    flowG: Graph[_Node],
    _s: _Node,
    _t: _Node,
    capacity: str = "capacity",
    flow_func: Callable[..., Incomplete] | None = None,
    **kwargs,
): ...
@_dispatchable
def minimum_cut_value(
    flowG: Graph[_Node],
    _s: _Node,
    _t: _Node,
    capacity: str = "capacity",
    flow_func: Callable[..., Incomplete] | None = None,
    **kwargs,
): ...



############################################################################
# Original work Copyright 2017 Palantir Technologies, Inc.                 #
# Original work licensed under the MIT License.                            #
# See ThirdPartyNotices.txt in the project root for license information.   #
# All modifications Copyright (c) Open Law Library. All rights reserved.   #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import copy
import logging
import os
import warnings
from typing import Dict, List, Optional, Union

from lsprotocol import types
from lsprotocol.types import (
    PositionEncodingKind,
    TextDocumentSyncKind,
    WorkspaceFolder,
)
from pygls.uris import to_fs_path, uri_scheme
from pygls.workspace.text_document import TextDocument
from pygls.workspace.position_codec import PositionCodec

logger = logging.getLogger(__name__)


class Workspace(object):
    def __init__(
        self,
        root_uri: Optional[str],
        sync_kind: TextDocumentSyncKind = TextDocumentSyncKind.Incremental,
        workspace_folders: Optional[List[WorkspaceFolder]] = None,
        position_encoding: Optional[
            Union[PositionEncodingKind, str]
        ] = PositionEncodingKind.Utf16,
    ):
        self._root_uri = root_uri
        if self._root_uri is not None:
            self._root_uri_scheme = uri_scheme(self._root_uri)
            root_path = to_fs_path(self._root_uri)
            if root_path is None:
                raise Exception("Couldn't get `root_path` from `root_uri`")
            self._root_path = root_path
        else:
            self._root_path = None
        self._sync_kind = sync_kind
        self._text_documents: Dict[str, TextDocument] = {}
        self._notebook_documents: Dict[str, types.NotebookDocument] = {}

        # Used to lookup notebooks which contain a given cell.
        self._cell_in_notebook: Dict[str, str] = {}
        self._folders: Dict[str, WorkspaceFolder] = {}
        self._docs: Dict[str, TextDocument] = {}
        self._position_encoding = position_encoding
        self._position_codec = PositionCodec(encoding=position_encoding)

        if workspace_folders is not None:
            for folder in workspace_folders:
                self.add_folder(folder)

    @property
    def position_encoding(self) -> Optional[Union[PositionEncodingKind, str]]:
        return self._position_encoding

    @property
    def position_codec(self) -> PositionCodec:
        return self._position_codec

    def _create_text_document(
        self,
        doc_uri: str,
        source: Optional[str] = None,
        version: Optional[int] = None,
        language_id: Optional[str] = None,
    ) -> TextDocument:
        return TextDocument(
            doc_uri,
            source=source,
            version=version,
            language_id=language_id,
            sync_kind=self._sync_kind,
            position_codec=self._position_codec,
        )

    def add_folder(self, folder: WorkspaceFolder):
        self._folders[folder.uri] = folder

    @property
    def documents(self):
        warnings.warn(
            "'workspace.documents' has been deprecated, use "
            "'workspace.text_documents' instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.text_documents

    @property
    def notebook_documents(self):
        return self._notebook_documents

    @property
    def text_documents(self):
        return self._text_documents

    @property
    def folders(self):
        return self._folders

    def get_notebook_document(
        self, *, notebook_uri: Optional[str] = None, cell_uri: Optional[str] = None
    ) -> Optional[types.NotebookDocument]:
        """Return the notebook corresponding with the given uri.

        If both ``notebook_uri`` and ``cell_uri`` are given, ``notebook_uri`` takes
        precedence.

        Parameters
        ----------
        notebook_uri
           If given, return the notebook document with the given uri.

        cell_uri
           If given, return the notebook document which contains a cell with the
           given uri

        Returns
        -------
        Optional[NotebookDocument]
           The requested notebook document if found, ``None`` otherwise.
        """
        if notebook_uri is not None:
            return self._notebook_documents.get(notebook_uri)

        if cell_uri is not None:
            notebook_uri = self._cell_in_notebook.get(cell_uri)
            if notebook_uri is None:
                return None

            return self._notebook_documents.get(notebook_uri)

        return None

    def get_text_document(self, doc_uri: str) -> TextDocument:
        """
        Return a managed document if-present,
        else create one pointing at disk.

        See https://github.com/Microsoft/language-server-protocol/issues/177
        """
        return self._text_documents.get(doc_uri) or self._create_text_document(doc_uri)

    def is_local(self):
        return (
            self._root_uri_scheme == "" or self._root_uri_scheme == "file"
        ) and os.path.exists(self._root_path)

    def put_notebook_document(self, params: types.DidOpenNotebookDocumentParams):
        notebook = params.notebook_document

        # Create a fresh instance to ensure our copy cannot be accidentally modified.
        self._notebook_documents[notebook.uri] = copy.deepcopy(notebook)

        for cell_document in params.cell_text_documents:
            self.put_text_document(cell_document, notebook_uri=notebook.uri)

    def put_text_document(
        self,
        text_document: types.TextDocumentItem,
        notebook_uri: Optional[str] = None,
    ):
        """Add a text document to the workspace.

        Parameters
        ----------
        text_document
           The text document to add

        notebook_uri
           If set, indicates that this text document represents a cell in a notebook
           document
        """
        doc_uri = text_document.uri

        self._text_documents[doc_uri] = self._create_text_document(
            doc_uri,
            source=text_document.text,
            version=text_document.version,
            language_id=text_document.language_id,
        )

        if notebook_uri:
            self._cell_in_notebook[doc_uri] = notebook_uri

    def remove_notebook_document(self, params: types.DidCloseNotebookDocumentParams):
        notebook_uri = params.notebook_document.uri
        self._notebook_documents.pop(notebook_uri, None)

        for cell_document in params.cell_text_documents:
            self.remove_text_document(cell_document.uri)

    def remove_text_document(self, doc_uri: str):
        self._text_documents.pop(doc_uri, None)
        self._cell_in_notebook.pop(doc_uri, None)

    def remove_folder(self, folder_uri: str):
        self._folders.pop(folder_uri, None)
        try:
            del self._folders[folder_uri]
        except KeyError:
            pass

    @property
    def root_path(self):
        return self._root_path

    @property
    def root_uri(self):
        return self._root_uri

    def update_notebook_document(self, params: types.DidChangeNotebookDocumentParams):
        uri = params.notebook_document.uri
        notebook = self._notebook_documents[uri]
        notebook.version = params.notebook_document.version

        if params.change.metadata:
            notebook.metadata = params.change.metadata

        cell_changes = params.change.cells
        if cell_changes is None:
            return

        # Process changes to any cell metadata.
        nb_cells = {cell.document: cell for cell in notebook.cells}
        for new_data in cell_changes.data or []:
            nb_cell = nb_cells.get(new_data.document)
            if nb_cell is None:
                logger.warning(
                    "Ignoring metadata for '%s': not in notebook.", new_data.document
                )
                continue

            nb_cell.kind = new_data.kind
            nb_cell.metadata = new_data.metadata
            nb_cell.execution_summary = new_data.execution_summary

        # Process changes to the notebook's structure
        structure = cell_changes.structure
        if structure:
            cells = notebook.cells
            new_cells = structure.array.cells or []

            # Re-order the cells
            before = cells[: structure.array.start]
            after = cells[(structure.array.start + structure.array.delete_count) :]
            notebook.cells = [*before, *new_cells, *after]

            for new_cell in structure.did_open or []:
                self.put_text_document(new_cell, notebook_uri=uri)

            for removed_cell in structure.did_close or []:
                self.remove_text_document(removed_cell.uri)

        # Process changes to the text content of existing cells.
        for text in cell_changes.text_content or []:
            for change in text.changes:
                self.update_text_document(text.document, change)

    def update_text_document(
        self,
        text_doc: types.VersionedTextDocumentIdentifier,
        change: types.TextDocumentContentChangeEvent,
    ):
        doc_uri = text_doc.uri
        self._text_documents[doc_uri].apply_change(change)
        self._text_documents[doc_uri].version = text_doc.version

    def get_document(self, *args, **kwargs):
        warnings.warn(
            "'workspace.get_document' has been deprecated, use "
            "'workspace.get_text_document' instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.get_text_document(*args, **kwargs)

    def remove_document(self, *args, **kwargs):
        warnings.warn(
            "'workspace.remove_document' has been deprecated, use "
            "'workspace.remove_text_document' instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.remove_text_document(*args, **kwargs)

    def put_document(self, *args, **kwargs):
        warnings.warn(
            "'workspace.put_document' has been deprecated, use "
            "'workspace.put_text_document' instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.put_text_document(*args, **kwargs)

    def update_document(self, *args, **kwargs):
        warnings.warn(
            "'workspace.update_document' has been deprecated, use "
            "'workspace.update_text_document' instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.update_text_document(*args, **kwargs)



import asyncio
from typing import AnyStr, Generic

from .expect import Expecter

async def expect_async(expecter: Expecter[AnyStr], timeout: float | None = None) -> int: ...
async def repl_run_command_async(repl, cmdlines, timeout: float | None = -1): ...

class PatternWaiter(asyncio.Protocol, Generic[AnyStr]):
    transport: asyncio.ReadTransport | None
    expecter: Expecter[AnyStr]
    fut: asyncio.Future[int]
    def set_expecter(self, expecter: Expecter[AnyStr]) -> None: ...
    def found(self, result: int) -> None: ...
    def error(self, exc: BaseException | type[BaseException]) -> None: ...
    def connection_made(self, transport: asyncio.BaseTransport) -> None: ...
    def data_received(self, data: bytes) -> None: ...
    def eof_received(self) -> None: ...
    def connection_lost(self, exc: BaseException | type[BaseException] | None) -> None: ...



class SimpleExtension:
    def GetExtensionVersion(self, vi) -> None: ...
    def HttpExtensionProc(self, control_block) -> int | None: ...
    def TerminateExtension(self, status) -> None: ...

class SimpleFilter:
    filter_flags: int | None
    def GetFilterVersion(self, fv) -> None: ...
    def HttpFilterProc(self, fc) -> None: ...
    def TerminateFilter(self, status) -> None: ...



import re
from _typeshed import Incomplete, StrPath
from collections.abc import Callable, Iterable
from io import TextIOWrapper
from pathlib import Path
from typing import ClassVar, Final, Literal, NoReturn, TypeVar, overload
from typing_extensions import deprecated

from docutils import nodes
from docutils.frontend import Values
from docutils.languages import _LanguageModule
from docutils.utils import Reporter
from docutils.writers import Writer as _Writer

_K = TypeVar("_K")
_V = TypeVar("_V")

__docformat__: Final = "reStructuredText"

LATEX_WRITER_DIR: Final[Path]

class Writer(_Writer[str]):
    default_template: ClassVar[str]
    default_template_path: ClassVar[Path]
    default_preamble: ClassVar[str]
    table_style_values: ClassVar[list[str]]
    relative_path_settings: ClassVar[tuple[str, ...]]
    settings_defaults: ClassVar[dict[str, int]]
    config_section: ClassVar[str]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    head_parts: ClassVar[tuple[str, ...]]
    visitor_attributes: ClassVar[tuple[str, ...]]
    translator_class: type[LaTeXTranslator]

class Babel:
    language_codes: ClassVar[dict[str, str]]
    warn_msg: ClassVar[str]
    active_chars: ClassVar[dict[str, str]]

    reporter: Reporter | None
    language: str
    otherlanguages: dict[str, str | bool]
    setup: list[str]

    def __init__(self, language_code: str, reporter: Reporter | None = None) -> None: ...
    def __call__(self) -> str: ...
    def language_name(self, language_code: str) -> str: ...
    def get_language(self) -> str: ...

@deprecated("Deprecated and will be removed in Docutils 0.24.")
class SortableDict(dict[_K, _V]):
    def sortedkeys(self) -> list[_K]: ...
    def sortedvalues(self) -> list[_V]: ...

class PreambleCmds:
    ch: ClassVar[str]
    color: ClassVar[str]
    float: ClassVar[str]
    linking: ClassVar[str]
    minitoc: ClassVar[str]
    table: ClassVar[str]
    table_columnwidth: ClassVar[str]
    textcomp: ClassVar[str]
    abstract_legacy: ClassVar[str]
    admonition_legacy: ClassVar[str]
    error_legacy: ClassVar[str]
    title_legacy: ClassVar[str]
    toc_list: ClassVar[str]
    ttem: ClassVar[str]
    duclass: ClassVar[str]
    providelength: ClassVar[str]
    abstract: ClassVar[str]
    dedication: ClassVar[str]
    docinfo: ClassVar[str]
    error: ClassVar[str]
    highlight_rules: ClassVar[str]
    admonition: ClassVar[str]
    fieldlist: ClassVar[str]
    footnotes: ClassVar[str]
    inline: ClassVar[str]
    legend: ClassVar[str]
    lineblock: ClassVar[str]
    optionlist: ClassVar[str]
    rubric: ClassVar[str]
    sidebar: ClassVar[str]
    title: ClassVar[str]
    subtitle: ClassVar[str]
    documentsubtitle: ClassVar[str]
    titlereference: ClassVar[str]
    transition: ClassVar[str]
    secnumdepth: ClassVar[str]
    rem: ClassVar[str]
    vmin: ClassVar[str]
    vmax: ClassVar[str]

fp: TextIOWrapper
line: str
block_name: str
definitions: str

class CharMaps:
    alltt: ClassVar[dict[int, str]]
    special: ClassVar[dict[int, str]]
    unsupported_unicode: ClassVar[dict[int, str]]
    utf8_supported_unicode: ClassVar[dict[int, str]]
    textcomp: ClassVar[dict[int, str]]
    pifont: ClassVar[dict[int, str]]

class DocumentClass:
    document_class: str
    sections: list[str]
    def __init__(self, document_class: str, with_part: bool = False) -> None: ...
    def section(self, level: int) -> str: ...
    def latex_section_depth(self, depth: int) -> int: ...

class Table:
    legacy_column_widths: bool
    caption: list[str]
    stubs: list[Incomplete]
    colwidths_auto: bool
    borders: str
    def __init__(self, translator: LaTeXTranslator, latex_type: str) -> None: ...
    def open(self) -> None: ...
    def close(self) -> None: ...
    def is_open(self) -> bool: ...
    def set_table_style(self, node, settings) -> None: ...
    def get_latex_type(self) -> str: ...
    def set(self, attr, value) -> None: ...
    def get(self, attr): ...
    def get_vertical_bar(self) -> Literal["|", ""]: ...
    def get_opening(self, width: str = r"\linewidth") -> str: ...
    def get_closing(self) -> str: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def get_colspecs(self, node: nodes.Element) -> str: ...
    def get_column_width(self) -> str: ...
    def get_multicolumn_width(self, start: int, len_: int) -> str: ...
    def need_recurse(self) -> bool | Literal[0]: ...
    def visit_thead(self) -> list[str]: ...
    def depart_thead(self) -> list[str]: ...
    def visit_row(self) -> None: ...
    def depart_row(self) -> list[str]: ...
    def set_rowspan(self, cell, value) -> None: ...
    def get_rowspan(self, cell): ...
    def get_entry_number(self) -> int: ...
    def visit_entry(self) -> None: ...
    def is_stub_column(self): ...

class LaTeXTranslator(nodes.NodeVisitor):
    is_xetex: ClassVar[bool]
    compound_enumerators: ClassVar[bool]
    section_prefix_for_enumerators: ClassVar[bool]
    section_enumerator_separator: ClassVar[str]
    has_latex_toc: ClassVar[bool]
    section_level: ClassVar[int]
    inside_citation_reference_label: ClassVar[bool]
    verbatim: ClassVar[bool]
    insert_non_breaking_blanks: ClassVar[bool]
    insert_newline: ClassVar[bool]
    literal: ClassVar[bool]
    alltt: ClassVar[bool]
    TITLEDATA_NODES: ClassVar[tuple[type[nodes.Element], ...]]

    warn: Callable[..., nodes.system_message]
    error: Callable[..., nodes.system_message]
    settings: Values
    latex_encoding: str
    use_latex_toc: Incomplete
    use_latex_docinfo: Incomplete
    use_latex_citations: Incomplete
    reference_label: Incomplete
    hyperlink_color: Incomplete
    font_encoding: str
    literal_block_env: str
    literal_block_options: str
    bibtex: Incomplete
    language_module: _LanguageModule
    babel: Babel
    author_separator: str
    documentoptions: str
    d_class: DocumentClass
    graphicx_package: str
    docutils_footnotes: Incomplete
    head_prefix: list[str]
    requirements: SortableDict[str, str] | list[str]
    latex_preamble: list[Incomplete]
    fallbacks: SortableDict[str, str] | list[str]
    pdfsetup: list[str]
    title: list[Incomplete]
    subtitle: list[Incomplete]
    titledata: list[Incomplete]
    body_pre_docinfo: list[Incomplete]
    docinfo: list[Incomplete]
    dedication: list[Incomplete]
    abstract: list[Incomplete]
    body: list[Incomplete]
    context: list[Incomplete]
    title_labels: list[Incomplete]
    subtitle_labels: list[Incomplete]
    author_stack: list[Incomplete]
    date: list[Incomplete]
    pdfauthor: list[Incomplete]
    pdfinfo: list[Incomplete]
    table_stack: list[Incomplete]
    active_table: Table
    out: list[Incomplete]
    out_stack: list[Incomplete]
    fallback_stylesheet: bool
    stylesheet: list[str]
    hyperref_options: str

    def __init__(self, document: nodes.document, babel_class: type = ...) -> None: ...
    def stylesheet_call(self, path: StrPath) -> str: ...
    def to_latex_encoding(self, docutils_encoding: str) -> str: ...
    def language_label(self, docutil_label: str) -> str: ...
    def encode(self, text: str) -> str: ...
    def attval(self, text: str, whitespace: re.Pattern[str] = ...) -> str: ...
    def is_inline(self, node: nodes.Node) -> bool: ...
    def ids_to_labels(
        self, node: nodes.Element, set_anchor: bool = True, protect: bool = False, newline: bool = False, pre_nl: bool = False
    ) -> list[str]: ...
    def append_hypertargets(self, node: nodes.Element) -> None: ...
    def set_align_from_classes(self, node) -> None: ...
    def insert_align_declaration(self, node: nodes.Element, default: str | None = None) -> None: ...
    def provide_fallback(self, feature: str, key: str | None = None) -> None: ...
    def duclass_open(self, node) -> None: ...
    def duclass_close(self, node) -> None: ...
    def push_output_collector(self, new_out) -> None: ...
    def pop_output_collector(self) -> None: ...
    def term_postfix(self, node: nodes.Element) -> str: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def depart_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def visit_acronym(self, node: nodes.acronym) -> None: ...
    def depart_acronym(self, node: nodes.acronym) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    def depart_admonition(self, node: nodes.admonition) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def depart_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def visit_comment(self, node: nodes.comment) -> None: ...
    def depart_comment(self, node: nodes.comment) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def depart_contact(self, node: nodes.contact) -> None: ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def depart_copyright(self, node: nodes.copyright) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def depart_date(self, node: nodes.date) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    @overload
    def visit_docinfo_item(self, node) -> None: ...
    @overload
    @deprecated("The `name` parameter is deprecated and will be removed in Docutils 0.24.")
    def visit_docinfo_item(self, node, name: str | None) -> None: ...
    def depart_docinfo_item(self, node) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def depart_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def make_title(self) -> None: ...
    def append_bibliogaphy(self) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def insert_additional_table_colum_delimiters(self) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def label_delim(self, node, bracket, superscript) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def to_latex_length(self, length_str: str, node: nodes.Node | None = None) -> str: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def depart_image(self, node: nodes.image) -> None: ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def is_plaintext(self, node) -> bool: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_meta(self, node: nodes.meta) -> None: ...
    def depart_meta(self, node: nodes.meta) -> None: ...
    def visit_math(self, node: nodes.math, math_env: str = "$") -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def depart_raw(self, node: nodes.raw) -> None: ...
    def has_unbalanced_braces(self, string: Iterable[str]) -> bool: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def depart_reference(self, node: nodes.reference) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    def depart_revision(self, node: nodes.revision) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_section(self, node: nodes.section) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...
    attribution_formats: dict[str, tuple[str, str]]
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def depart_status(self, node: nodes.status) -> None: ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_substitution_reference(self, node: nodes.substitution_reference) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def depart_target(self, node: nodes.target) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def thead_depth(self) -> int: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def visit_title(self, node: nodes.title) -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_contents(self, node) -> None: ...
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def depart_version(self, node: nodes.version) -> None: ...
    def unimplemented_visit(self, node: nodes.Node) -> NoReturn: ...



from antlr4.error.Errors import IllegalStateException as IllegalStateException
from antlr4.Lexer import Lexer as ActualLexer, TokenSource
from antlr4.Token import Token as Token

Lexer: None

class TokenStream: ...

class BufferedTokenStream(TokenStream):
    __slots__ = ("tokenSource", "tokens", "index", "fetchedEOF")
    tokenSource: TokenSource
    tokens: list[Token]
    index: int
    fetchedEOF: bool
    def __init__(self, tokenSource: ActualLexer | None) -> None: ...
    def mark(self) -> int: ...
    def release(self, marker: int) -> None: ...
    def reset(self) -> None: ...
    def seek(self, index: int) -> None: ...
    def get(self, index: int) -> Token: ...
    def consume(self) -> None: ...
    def sync(self, i: int) -> bool: ...
    def fetch(self, n: int) -> int: ...
    def getTokens(self, start: int, stop: int, types: set[int] | None = None) -> list[Token]: ...
    def LA(self, i: int) -> int: ...
    def LB(self, k: int) -> Token | None: ...
    def LT(self, k: int) -> Token | None: ...
    def adjustSeekIndex(self, i: int) -> int: ...
    def lazyInit(self) -> None: ...
    def setup(self) -> None: ...
    def setTokenSource(self, tokenSource: ActualLexer | None) -> None: ...
    def nextTokenOnChannel(self, i: int, channel: int) -> int: ...
    def previousTokenOnChannel(self, i: int, channel: int) -> int: ...
    def getHiddenTokensToRight(self, tokenIndex: int, channel: int = -1) -> list[Token] | None: ...
    def getHiddenTokensToLeft(self, tokenIndex: int, channel: int = -1) -> list[Token] | None: ...
    def filterForChannel(self, left: int, right: int, channel: int) -> list[Token] | None: ...
    def getSourceName(self) -> str: ...
    def getText(self, start: int | None = None, stop: int | None = None) -> str: ...
    def fill(self) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol messages for describing the results of benchmarks and unit tests."""

import google.protobuf.descriptor
from tensorflow.compiler.xla.tsl.protobuf.test_log_pb2 import (
    AvailableDeviceInfo as AvailableDeviceInfo,
    BenchmarkEntries as BenchmarkEntries,
    BenchmarkEntry as BenchmarkEntry,
    BuildConfiguration as BuildConfiguration,
    CommitId as CommitId,
    CPUInfo as CPUInfo,
    EntryValue as EntryValue,
    GPUInfo as GPUInfo,
    MachineConfiguration as MachineConfiguration,
    MemoryInfo as MemoryInfo,
    MetricEntry as MetricEntry,
    PlatformInfo as PlatformInfo,
    RunConfiguration as RunConfiguration,
    TestResults as TestResults,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



class RequestAborted(Exception): ...
class RequestTimeout(RequestAborted): ...
class InvalidChannelLayerError(ValueError): ...
class AcceptConnection(Exception): ...
class DenyConnection(Exception): ...
class ChannelFull(Exception): ...
class MessageTooLarge(Exception): ...
class StopConsumer(Exception): ...



from collections.abc import Generator, Iterable
from typing import Final, Literal
from typing_extensions import TypeAlias

from docutils import ApplicationError

_TokenNames: TypeAlias = Literal["long", "short", "none"]

__docformat__: Final = "reStructuredText"
with_pygments: bool
unstyled_tokens: list[str]

class LexerError(ApplicationError): ...

class Lexer:
    code: str
    language: str
    tokennames: _TokenNames
    lexer: Lexer | None
    def __init__(self, code: str, language: str, tokennames: _TokenNames = "short") -> None: ...
    def merge(self, tokens: Iterable[tuple[_TokenNames, str]]) -> Generator[tuple[_TokenNames, str]]: ...
    def __iter__(self) -> Generator[tuple[list[str], str]]: ...

class NumberLines:
    tokens: Iterable[tuple[list[str], str]]
    startline: int
    fmt_str: str
    def __init__(self, tokens: Iterable[tuple[list[str], str]], startline: int, endline: int) -> None: ...
    def __iter__(self) -> Generator[tuple[list[str], str]]: ...



from collections.abc import Callable
from typing import overload

from pymeeus.Angle import Angle

class CurveFitting:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, a: CurveFitting, /) -> None: ...
    @overload
    def __init__(self, a: list[float | Angle] | tuple[float | Angle, ...], /) -> None: ...
    @overload
    def __init__(
        self, a1: list[float | Angle] | tuple[float | Angle, ...], a2: list[float | Angle] | tuple[float | Angle, ...], /
    ) -> None: ...
    @overload
    def __init__(
        self, a1: float | Angle, a2: float | Angle, a3: float | Angle, a4: float | Angle, /, *rest: float | Angle
    ) -> None: ...
    @overload
    def set(self) -> None: ...
    @overload
    def set(self, a: CurveFitting, /) -> None: ...
    @overload
    def set(self, a: list[float | Angle] | tuple[float | Angle, ...], /) -> None: ...
    @overload
    def set(
        self, a1: list[float | Angle] | tuple[float | Angle, ...], a2: list[float | Angle] | tuple[float | Angle, ...], /
    ) -> None: ...
    @overload
    def set(
        self, a1: float | Angle, a2: float | Angle, a3: float | Angle, a4: float | Angle, /, *rest: float | Angle
    ) -> None: ...
    def __len__(self) -> int: ...
    def correlation_coeff(self) -> float: ...
    def linear_fitting(self) -> tuple[float, float]: ...
    def quadratic_fitting(self) -> tuple[float, float, float]: ...
    def general_fitting(
        self, f0: Callable[..., float], f1: Callable[..., float] = ..., f2: Callable[..., float] = ...
    ) -> tuple[float, float, float]: ...

def main() -> None: ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Logs:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def search(
        self,
        page: int = 0,
        per_page: int = 50,
        sort: str | None = None,
        q: str | None = None,
        include_totals: bool = True,
        fields: list[str] | None = None,
        from_param: str | None = None,
        take: int | None = None,
        include_fields: bool = True,
    ) -> dict[str, Incomplete]: ...
    async def search_async(
        self,
        page: int = 0,
        per_page: int = 50,
        sort: str | None = None,
        q: str | None = None,
        include_totals: bool = True,
        fields: list[str] | None = None,
        from_param: str | None = None,
        take: int | None = None,
        include_fields: bool = True,
    ) -> dict[str, Incomplete]: ...
    def get(self, id: str) -> dict[str, Incomplete]: ...
    async def get_async(self, id: str) -> dict[str, Incomplete]: ...
